@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.c,v 1.49 2003/12/11 18:07:39 fuyu Exp $
+ * $Id: bbs_thread_ui.c,v 1.50 2003/12/12 21:43:52 fuyu Exp $
  */
 
 #include "config.h"
@@ -39,6 +39,8 @@
 #include "bulletin_board_ui.h"
 #include "icon_label.h"
 
+#include "regex_utils.h"
+
 #include "htmlutils.h"
 #include "worker.h"
 #include "utils.h"
@@ -169,10 +171,13 @@ typedef struct _RendererContext
   GtkTextTag *response_paragraph_tag;
   GtkTextTag *bold_tag;
 
-#if defined(ENABLE_ONIGURUMA)
+  gboolean transparent_a_bone;
+  gpointer a_bone_by_name;
+  gpointer a_bone_by_id;
+  gpointer a_bone_by_content;
+
   regex_t comma_separated_link_pattern;
   regex_t informal_link_pattern;
-#endif
 } RendererContext;
 
 
@@ -203,57 +208,6 @@ struct _PopupContext
 
 
 /* PopupϢ */
-#if !ENABLE_MULTIPLE_POPUPS
-gchar popup_renderer_default_text_buffer[DEFAULT_BUFFER_SIZE];
-PopupContext global_popup_context =
-{
-  {
-    {
-      0,
-      start_element_cb,
-      end_element_cb,
-      NULL,
-      characters_cb
-    },		/* handler */
-    NULL,	/* view */
-    NULL,	/* thread */
-    NULL,	/* application */
-    -1,		/* last_read_number: ְ㤨ƥޡդʤ褦 */
-  
-    NULL,				/* link */
-    popup_renderer_default_text_buffer,	/* link_text_buffer */
-    DEFAULT_BUFFER_SIZE,		/* link_text_buffer_size */
-    0,					/* link_text_length */
-
-    0,					/* number_of_gt_characters */
-    FALSE,				/* last_text_is_link */
-    FALSE,				/* show_mailto_literally */
-
-    NULL,				/* tag_table */
-
-    NULL,				/* thread_title_tag */
-    NULL,				/* response_header_tag */
-    NULL,				/* response_name_tag */
-    NULL,				/* response_paragraph_tag */
-    NULL,				/* bold_tag */
-  },
-
-  NULL,		/* popup_window */
-  NULL,		/* popup_frame */
-
-  NULL,		/* parent_context */
-
-  NULL,		/* url */
-  0,		/* delay_id */
-  0,		/* close_delay_id */
-  0,		/* x_pos */
-  0,		/* y_pos */
-
-  FALSE,	/* now_pointed */
-};
-#endif
-
-
 enum {
   LINK_POPUP_MENU_OPEN_LINK = 1,
   LINK_POPUP_MENU_OPEN_LINK_IN_TAB,
@@ -605,10 +559,6 @@ initialize_thread_ui(OchushaApplication *application)
   if (application->thread_view_font_name != NULL
       && *application->thread_view_font_name != '\0')
     set_thread_view_font(application->thread_view_font_name);
-
-#if !ENABLE_MULTIPLE_POPUPS
-  initialize_popup_context(&global_popup_context, application);
-#endif
 }
 
 
@@ -633,6 +583,24 @@ bbs_thread_info_free(BBSThreadGUIInfo *info)
       info->last_mail = NULL;
     }
 
+  if (info->a_bone_by_name_pattern != NULL)
+    {
+      G_FREE(info->a_bone_by_name_pattern);
+      info->a_bone_by_name_pattern = NULL;
+    }
+
+  if (info->a_bone_by_id_pattern != NULL)
+    {
+      G_FREE(info->a_bone_by_id_pattern);
+      info->a_bone_by_id_pattern = NULL;
+    }
+
+  if (info->a_bone_by_content_pattern != NULL)
+    {
+      G_FREE(info->a_bone_by_content_pattern);
+      info->a_bone_by_content_pattern = NULL;
+    }
+
   if (info->write_dialog != NULL)
     gtk_widget_destroy(info->write_dialog);
 
@@ -857,11 +825,27 @@ popup_context_free(PopupContext *popup_context)
       popup_context->renderer_context.link_text_buffer = NULL;
     }
 
-#if defined(ENABLE_ONIGURUMA)
   /* "\357\274\214\343\200\201"  "" UTF-8ɽ */
   regfree(&popup_context->renderer_context.comma_separated_link_pattern);
   regfree(&popup_context->renderer_context.informal_link_pattern);
-#endif
+
+  if (popup_context->renderer_context.a_bone_by_name != NULL)
+    {
+      oniguruma_regex_free(popup_context->renderer_context.a_bone_by_name);
+      popup_context->renderer_context.a_bone_by_name = NULL;
+    }
+
+  if (popup_context->renderer_context.a_bone_by_id != NULL)
+    {
+      oniguruma_regex_free(popup_context->renderer_context.a_bone_by_id);
+      popup_context->renderer_context.a_bone_by_id = NULL;
+    }
+
+  if (popup_context->renderer_context.a_bone_by_content != NULL)
+    {
+      oniguruma_regex_free(popup_context->renderer_context.a_bone_by_content);
+      popup_context->renderer_context.a_bone_by_content = NULL;
+    }
 
   if (popup_context->url != NULL)
     {
@@ -895,9 +879,9 @@ popup_context_free(PopupContext *popup_context)
  * MEMO: gdk_threads_enter()ĶƤӽФȡ
  */
 static PopupContext *
-ensure_popup_context(BBSThreadView *parent, OchushaApplication *application)
+ensure_popup_context(BBSThreadView *parent, OchushaBBSThread *thread,
+		     OchushaApplication *application)
 {
-#if ENABLE_MULTIPLE_POPUPS
   RendererContext *renderer_context;
   PopupContext *popup_context = g_object_get_qdata(G_OBJECT(parent),
 						   popup_context_id);
@@ -930,8 +914,6 @@ ensure_popup_context(BBSThreadView *parent, OchushaApplication *application)
   g_object_set_qdata_full(G_OBJECT(parent), popup_context_id, popup_context,
 			  (GDestroyNotify)popup_context_free);
 
-
-#if defined(ENABLE_ONIGURUMA)
   /* "\357\274\214\343\200\201"  "" UTF-8ɽ */
   if (regcomp(&renderer_context->comma_separated_link_pattern,
 	      "^[,\357\274\214\343\200\201][ ]*([1-9][0-9]*)(-[1-9][0-9]*)?",
@@ -947,12 +929,84 @@ ensure_popup_context(BBSThreadView *parent, OchushaApplication *application)
     {
       fprintf(stderr, "invalid regular expression\n");
     }
-#endif
+
+  if (application->disable_popup_a_bone)
+    {
+      gboolean a_bone_by_name;
+      gchar *a_bone_by_name_pattern;
+      gboolean a_bone_by_id;
+      gchar *a_bone_by_id_pattern;
+      gboolean a_bone_by_content;
+      gchar *a_bone_by_content_pattern;
+
+      BBSThreadGUIInfo *thread_info = ensure_bbs_thread_info(thread);
+
+      if (thread_info->thread_local_a_bone)
+	{
+	  a_bone_by_name = thread_info->a_bone_by_name;
+	  a_bone_by_name_pattern = thread_info->a_bone_by_name_pattern;
+	  a_bone_by_id = thread_info->a_bone_by_id;
+	  a_bone_by_id_pattern = thread_info->a_bone_by_id_pattern;
+	  a_bone_by_content = thread_info->a_bone_by_content;
+	  a_bone_by_content_pattern = thread_info->a_bone_by_content_pattern;
+	}
+      else
+	{
+	  BulletinBoardGUIInfo *board_info
+	    = ensure_bulletin_board_info(thread->board, application);
+	  if (board_info->board_local_a_bone)
+	    {
+	      a_bone_by_name = board_info->a_bone_by_name;
+	      a_bone_by_name_pattern = board_info->a_bone_by_name_pattern;
+	      a_bone_by_id = board_info->a_bone_by_id;
+	      a_bone_by_id_pattern = board_info->a_bone_by_id_pattern;
+	      a_bone_by_content = board_info->a_bone_by_content;
+	      a_bone_by_content_pattern
+		= board_info->a_bone_by_content_pattern;
+	    }
+	  else
+	    {
+	      a_bone_by_name = application->a_bone_by_name;
+	      a_bone_by_name_pattern = application->a_bone_by_name_pattern;
+	      a_bone_by_id = application->a_bone_by_id;
+	      a_bone_by_id_pattern = application->a_bone_by_id_pattern;
+	      a_bone_by_content = application->a_bone_by_content;
+	      a_bone_by_content_pattern
+		= application->a_bone_by_content_pattern;
+	    }
+	}
+
+      renderer_context->transparent_a_bone = TRUE;
+      if (a_bone_by_name
+	  && a_bone_by_name_pattern != NULL && *a_bone_by_name_pattern != '\0')
+	renderer_context->a_bone_by_name
+	  = oniguruma_regex_new(a_bone_by_name_pattern, NULL);
+      if (a_bone_by_id
+	  && a_bone_by_id_pattern != NULL && *a_bone_by_id_pattern != '\0')
+	renderer_context->a_bone_by_id
+	  = oniguruma_regex_new(a_bone_by_id_pattern, NULL);
+      if (a_bone_by_content
+	  && a_bone_by_content_pattern != NULL
+	  && *a_bone_by_content_pattern != '\0')
+	{
+	  const char *encoding
+	    = ochusha_bbs_thread_get_response_character_encoding(thread);
+	  iconv_t converter = iconv_open(encoding, "UTF-8");
+	  if (converter != (iconv_t)-1)
+	    {
+	      char *pattern
+		= convert_string(converter, NULL,
+				 a_bone_by_content_pattern, -1);
+	      if (pattern != NULL && *pattern != '\0')
+		renderer_context->a_bone_by_content
+		  = oniguruma_regex_new(pattern, encoding);
+	      iconv_close(converter);
+	      G_FREE(pattern);
+	    }
+	}
+    }
 
   return popup_context;
-#else
-  return &global_popup_context;
-#endif
 }
 
 
@@ -964,7 +1018,7 @@ link_mouse_out_cb(BBSThreadView *view, GdkEventAny *event,
   PopupContext *popup_context;
   GtkStatusbar *statusbar = application->statusbar;
 
-  popup_context = ensure_popup_context(view, application);
+  popup_context = ensure_popup_context(view, thread, application);
 
   if (message_id != 0)
     {
@@ -1133,7 +1187,13 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
       bbs_thread_view_get_tag_by_name(job_args->view, "bold"),
   };
 
-#if defined(ENABLE_ONIGURUMA)
+  gboolean a_bone_by_name;
+  gchar *a_bone_by_name_pattern;
+  gboolean a_bone_by_id;
+  gchar *a_bone_by_id_pattern;
+  gboolean a_bone_by_content;
+  gchar *a_bone_by_content_pattern;
+
   /* "\357\274\214\343\200\201\357\274\235"  "" UTF-8ɽ */
   if (regcomp(&context.comma_separated_link_pattern,
 	      "^[,=\357\274\214\343\200\201\357\274\235][ ]*([1-9][0-9]*)(-[1-9][0-9]*)?",
@@ -1149,7 +1209,66 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
     {
       fprintf(stderr, "invalid regular expression\n");
     }
-#endif
+
+  if (info->thread_local_a_bone)
+    {
+      a_bone_by_name = info->a_bone_by_name;
+      a_bone_by_name_pattern = info->a_bone_by_name_pattern;
+      a_bone_by_id = info->a_bone_by_id;
+      a_bone_by_id_pattern = info->a_bone_by_id_pattern;
+      a_bone_by_content = info->a_bone_by_content;
+      a_bone_by_content_pattern = info->a_bone_by_content_pattern;
+    }
+  else
+    {
+      BulletinBoardGUIInfo *board_info
+	= ensure_bulletin_board_info(thread->board, application);
+      if (board_info->board_local_a_bone)
+	{
+	  a_bone_by_name = board_info->a_bone_by_name;
+	  a_bone_by_name_pattern = board_info->a_bone_by_name_pattern;
+	  a_bone_by_id = board_info->a_bone_by_id;
+	  a_bone_by_id_pattern = board_info->a_bone_by_id_pattern;
+	  a_bone_by_content = board_info->a_bone_by_content;
+	  a_bone_by_content_pattern = board_info->a_bone_by_content_pattern;
+	}
+      else
+	{
+	  a_bone_by_name = application->a_bone_by_name;
+	  a_bone_by_name_pattern = application->a_bone_by_name_pattern;
+	  a_bone_by_id = application->a_bone_by_id;
+	  a_bone_by_id_pattern = application->a_bone_by_id_pattern;
+	  a_bone_by_content = application->a_bone_by_content;
+	  a_bone_by_content_pattern = application->a_bone_by_content_pattern;
+	}
+    }
+
+  context.transparent_a_bone = application->enable_transparent_a_bone;
+  if (a_bone_by_name
+      && a_bone_by_name_pattern != NULL && *a_bone_by_name_pattern != '\0')
+    context.a_bone_by_name = oniguruma_regex_new(a_bone_by_name_pattern, NULL);
+  if (a_bone_by_id
+      && a_bone_by_id_pattern != NULL && *a_bone_by_id_pattern != '\0')
+    context.a_bone_by_id = oniguruma_regex_new(a_bone_by_id_pattern, NULL);
+  if (a_bone_by_content
+      && a_bone_by_content_pattern != NULL
+      && *a_bone_by_content_pattern != '\0')
+    {
+      const char *encoding
+	= ochusha_bbs_thread_get_response_character_encoding(thread);
+      iconv_t converter = iconv_open(encoding, "UTF-8");
+      if (converter != (iconv_t)-1)
+	{
+	  char *pattern
+	    = convert_string(converter, NULL,
+			     a_bone_by_content_pattern, -1);
+	  if (pattern != NULL && *pattern != '\0')
+	    context.a_bone_by_content = oniguruma_regex_new(pattern,
+							    encoding);
+	  iconv_close(converter);
+	  G_FREE(pattern);
+	}
+    }
 
   if (info->offsets == NULL)
     {
@@ -1251,11 +1370,27 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
   }
   BBS_THREAD_UI_UNLOCK;
 
-#if defined(ENABLE_ONIGURUMA)
   /* "\357\274\214\343\200\201"  "" UTF-8ɽ */
   regfree(&context.comma_separated_link_pattern);
   regfree(&context.informal_link_pattern);
-#endif
+
+  if (context.a_bone_by_name != NULL)
+    {
+      oniguruma_regex_free(context.a_bone_by_name);
+      context.a_bone_by_name = NULL;
+    }
+
+  if (context.a_bone_by_id != NULL)
+    {
+      oniguruma_regex_free(context.a_bone_by_id);
+      context.a_bone_by_id = NULL;
+    }
+
+  if (context.a_bone_by_content != NULL)
+    {
+      oniguruma_regex_free(context.a_bone_by_content);
+      context.a_bone_by_content = NULL;
+    }
 
   if (context.link_text_buffer != default_text_buffer)
     G_FREE(context.link_text_buffer);
@@ -1333,12 +1468,26 @@ render_response(OchushaBBSThread *thread, gint number,
   BBSThreadView *view = context->view;
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   char buffer[13];
-  const char *cur_pos;
+  gboolean a_bone;
+
 #if DEBUG_GUI_MOST
   fprintf(stderr, "rendering response #%d, last_read_number=%d\n",
 	  number, context->last_read_number);
 #endif
 
+  if (context->a_bone_by_name != NULL
+      && oniguruma_regex_match(context->a_bone_by_name, response->name))
+    a_bone = TRUE;
+  else if (context->a_bone_by_id != NULL
+	   && oniguruma_regex_match(context->a_bone_by_id, response->date_id))
+    a_bone = TRUE;
+  else if (context->a_bone_by_content != NULL
+	   && oniguruma_regex_match(context->a_bone_by_content,
+				    response->content))
+    a_bone = TRUE;
+  else
+    a_bone = FALSE;
+
   if (context->last_read_number >= 0)
     {
       if (number >= info->offsets_length)
@@ -1365,6 +1514,9 @@ render_response(OchushaBBSThread *thread, gint number,
       update_threadlist_entry(context->application, thread);
     }
 
+  if (a_bone && context->transparent_a_bone)
+    return TRUE;
+
   snprintf(buffer, 13, "\n%d", number);
 
   gdk_threads_enter();
@@ -1384,88 +1536,121 @@ render_response(OchushaBBSThread *thread, gint number,
   bbs_thread_view_append_text(view, buffer, -1);
   bbs_thread_view_append_text(view, LARGE_COLON, 3);
 
-  if (response->mailto == NULL)
+  if (!a_bone)
     {
-      bbs_thread_view_push_tag(view, context->response_name_tag);
-      bbs_thread_view_push_tag(view, context->bold_tag);
-      parse_text(&context->handler, context, response->name, -1);
-      bbs_thread_view_pop_tags(view, context->bold_tag);
-      bbs_thread_view_pop_tags(view, context->response_name_tag);
-      if (context->show_mailto_literally)
-	bbs_thread_view_append_text(view, " []", 3);
+      if (response->mailto == NULL)
+	{
+	  bbs_thread_view_push_tag(view, context->response_name_tag);
+	  bbs_thread_view_push_tag(view, context->bold_tag);
+	  parse_text(&context->handler, context, response->name, -1);
+	  bbs_thread_view_pop_tags(view, context->bold_tag);
+	  bbs_thread_view_pop_tags(view, context->response_name_tag);
+	  if (context->show_mailto_literally)
+	    bbs_thread_view_append_text(view, " []", 3);
+	}
+      else
+	{
+	  gchar *name = simple_string_canon(response->name, -1,
+					    context->handler.converter,
+					    context->handler.helper);
+	  gchar *mailto = convert_string(context->handler.converter,
+					 context->handler.helper,
+					 response->mailto, -1);
+	  bbs_thread_view_push_tag(view, context->response_name_tag);
+	  if (strcmp(response->mailto, "sage") != 0)
+	    bbs_thread_view_append_text_as_link(view, name, -1,
+						(mailto != NULL
+						 ? mailto : _("broken here")));
+	  else
+	    bbs_thread_view_append_text_as_sage(view, name, -1,
+						(mailto != NULL
+						 ? mailto : _("broken here")));
+	  bbs_thread_view_pop_tags(view, context->response_name_tag);
+	  if (context->show_mailto_literally)
+	    {
+	      char *tmp_mailto;
+	      bbs_thread_view_append_text(view, " [", 2);
+	      
+	      if (mailto != NULL)
+		tmp_mailto = simple_string_canon(mailto, -1, NULL, NULL);
+	      else
+		tmp_mailto = NULL;
+
+	      if (tmp_mailto != NULL)
+		{
+		  bbs_thread_view_append_text(view, tmp_mailto, -1);
+		  G_FREE(tmp_mailto);
+		}
+	      else
+		bbs_thread_view_append_text(view, _("broken here"), -1);
+
+	      bbs_thread_view_append_text(view, "]", 1);
+	    }
+	    
+	  if (name != NULL)
+	    G_FREE(name);
+	  if (mailto != NULL)
+	    G_FREE(mailto);
+	}
     }
   else
     {
-      gchar *name = simple_string_canon(response->name, -1,
-					context->handler.converter,
-					context->handler.helper);
-      gchar *mailto = convert_string(context->handler.converter,
-				     context->handler.helper,
-				     response->mailto, -1);
+      /* ܡ */
       bbs_thread_view_push_tag(view, context->response_name_tag);
-      if (strcmp(response->mailto, "sage") != 0)
-	bbs_thread_view_append_text_as_link(view, name, -1,
-					    (mailto != NULL
-					     ? mailto : _("broken here")));
-      else
-	bbs_thread_view_append_text_as_sage(view, name, -1,
-					    (mailto != NULL
-					     ? mailto : _("broken here")));
+      bbs_thread_view_push_tag(view, context->bold_tag);
+      bbs_thread_view_append_text_as_link(view, _("A Bone"), -1, _("A Bone"));
+      bbs_thread_view_pop_tags(view, context->bold_tag);
       bbs_thread_view_pop_tags(view, context->response_name_tag);
       if (context->show_mailto_literally)
 	{
-	  char *tmp_mailto;
 	  bbs_thread_view_append_text(view, " [", 2);
-
-	  if (mailto != NULL)
-	    tmp_mailto = simple_string_canon(mailto, -1, NULL, NULL);
-	  else
-	    tmp_mailto = NULL;
-
-	  if (tmp_mailto != NULL)
-	    {
-	      bbs_thread_view_append_text(view, tmp_mailto, -1);
-	      G_FREE(tmp_mailto);
-	    }
-	  else
-	    bbs_thread_view_append_text(view, _("broken here"), -1);
-
+	  bbs_thread_view_append_text(view, _("A Bone"), -1);
 	  bbs_thread_view_append_text(view, "]", 1);
 	}
-	    
-      if (name != NULL)
-	G_FREE(name);
-      if (mailto != NULL)
-	G_FREE(mailto);
     }
 
   bbs_thread_view_append_text(view, LARGE_COLON, 3);
-  parse_text(&context->handler, context, response->date_id, -1);
+  if (!a_bone)
+    parse_text(&context->handler, context, response->date_id, -1);
+  else
+    bbs_thread_view_append_text(view, _("A Bone"), -1);
 
   bbs_thread_view_append_text(view, "\n", 1);
   bbs_thread_view_pop_tags(view, context->response_header_tag);
 
   bbs_thread_view_push_tag(view, context->response_paragraph_tag);
-  cur_pos = response->content;
-  while (cur_pos != NULL)
+
+  if (!a_bone)
     {
-      cur_pos = parse_text(&context->handler, context, cur_pos, -1);
-      if (cur_pos != NULL)
+      const char *cur_pos = response->content;
+      while (cur_pos != NULL)
 	{
-	  if (*cur_pos == '&')
+	  cur_pos = parse_text(&context->handler, context, cur_pos, -1);
+	  if (cur_pos != NULL)
 	    {
-	      parse_text(&context->handler, context, "&amp;", 5);
-	      cur_pos++;
+	      if (*cur_pos == '&')
+		{
+		  parse_text(&context->handler, context, "&amp;", 5);
+		  cur_pos++;
+		}
+	      else if (*cur_pos == '<')
+		{
+		  parse_text(&context->handler, context, "&lt;", 4);
+		  cur_pos++;
+		}
+	      else
+		break;	/*  */
 	    }
-	  else if (*cur_pos == '<')
-	    {
-	      parse_text(&context->handler, context, "&lt;", 4);
-	      cur_pos++;
-	    }
-	  else
-	    break;	/*  */
 	}
     }
+  else
+    {
+      /* ܡ */
+      gchar *link = ochusha_bbs_thread_get_url_for_response(context->thread,
+							    number, number);
+      bbs_thread_view_append_text_as_link(view, _("A Bone"), -1, link);
+      G_FREE(link);
+    }
 
   bbs_thread_view_pop_tags(view, context->response_paragraph_tag);
 
@@ -1718,7 +1903,6 @@ characters_cb(void *user_data, const char *ch, int len)
 
   if (context->link == NULL)
     {
-#if defined(ENABLE_ONIGURUMA)
       if (context->application->analyze_informal_links)
 	{
 	  regmatch_t match[3];
@@ -1733,32 +1917,17 @@ characters_cb(void *user_data, const char *ch, int len)
 	    }
 	  else if (context->number_of_gt_characters > 0)
 	    {
-#if 0
-	      if (len > 0)
-		fprintf(stderr, "XXX: text=\"%s\", len=%d\n", ch, len);
-#endif
 	      text = G_MALLOC(len + context->number_of_gt_characters + 1);
 	      memset(text, '>', context->number_of_gt_characters);
 	      memcpy(text + context->number_of_gt_characters, ch, len);
-#if 0
-	      strncpy(text + context->number_of_gt_characters, ch, len);
-#endif
 	      text_top = text;
 	      len += context->number_of_gt_characters;
 	      text[len] = '\0';
 	      context->number_of_gt_characters = 0;
-#if 0
-	      if (len > 0)
-		fprintf(stderr, "YYY: text=\"%s\", len=%d\n", text, len);
-#endif
 	    }
 	  else
 	    text = g_strndup(ch, len);
 
-#if 0
-	  if (len > 0)
-	    fprintf(stderr, "ZZZ: text=\"%s\", len=%d\n", text, len);
-#endif
 	  text_top = text;
 
 	  while (len > 0)
@@ -1789,15 +1958,6 @@ characters_cb(void *user_data, const char *ch, int len)
 		    }
 		  else
 		    to = from;
-#if 0
-		  if (match[1].rm_so == -1 || match[1].rm_eo == match[1].rm_so
-		      || sscanf(text_top + match[1].rm_so, "%d", &from) != 1)
-		    from = 0;
-
-		  if (match[2].rm_so == -1 || match[2].rm_eo == match[2].rm_so
-		      || sscanf(text_top + match[2].rm_so + 1, "%d", &to) != 1)
-		    to = from;
-#endif
 
 		  if (from > 0)
 		    link = ochusha_bbs_thread_get_url_for_response(context->thread,
@@ -1811,7 +1971,7 @@ characters_cb(void *user_data, const char *ch, int len)
 							  text_top + match[0].rm_so,
 							  match_len,
 							  link);
-		      g_free(link);
+		      G_FREE(link);
 		    }
 		  else
 		    bbs_thread_view_append_text(context->view,
@@ -1836,11 +1996,6 @@ characters_cb(void *user_data, const char *ch, int len)
 		  int to;
 		  char *link;
 
-#if 0
-		  fprintf(stderr, "ZZZ: len=%d, no_match_len=%d, match_len=%d\n",
-			  len, no_match_len, match_len);
-#endif
-
 		  if (no_match_len > 0)
 		    bbs_thread_view_append_text(context->view, text_top,
 						no_match_len);
@@ -1862,15 +2017,6 @@ characters_cb(void *user_data, const char *ch, int len)
 		    }
 		  else
 		    to = from;
-#if 0
-		  if (match[1].rm_so == -1 || match[1].rm_eo == match[1].rm_so
-		      || sscanf(text_top + match[1].rm_so, "%d", &from) != 1)
-		    from = 0;
-
-		  if (match[2].rm_so == -1 || match[2].rm_eo == match[2].rm_so
-		      || sscanf(text_top + match[2].rm_so + 1, "%d", &to) != 1)
-		    to = from;
-#endif
 
 		  if (from > 0)
 		    link = ochusha_bbs_thread_get_url_for_response(context->thread,
@@ -1884,7 +2030,7 @@ characters_cb(void *user_data, const char *ch, int len)
 							  text_top + match[0].rm_so,
 							  match_len,
 							  link);
-		      g_free(link);
+		      G_FREE(link);
 		      context->last_text_is_link = TRUE;
 		    }
 		  else
@@ -1905,15 +2051,8 @@ characters_cb(void *user_data, const char *ch, int len)
 
 		  if (len > 0)
 		    {
-#if 0
-		      fprintf(stderr, "CHECKPOINT #1\n");
-		      fprintf(stderr, "Appending text: \"%s\", len=%d\n",
-			      text_top, len);
-#endif
-		      bbs_thread_view_append_text(context->view, text_top, len);
-#if 0
-		      fprintf(stderr, "CHECKPOINT #2\n");
-#endif
+		      bbs_thread_view_append_text(context->view,
+						  text_top, len);
 		    }
 
 		  break;
@@ -1923,9 +2062,6 @@ characters_cb(void *user_data, const char *ch, int len)
 	  if (text != NULL)
 	    g_free(text);
 	}
-#else
-      bbs_thread_view_append_text(context->view, ch, len);
-#endif	/* defined(ENABLE_ONIGURUMA) */
       return;
     }
 
@@ -2251,7 +2387,7 @@ show_popup(OchushaApplication *application, OchushaBBSThread *thread,
   gint response_to = 0;
   gint num_responses;
 
-  popup_context = ensure_popup_context(parent, application);
+  popup_context = ensure_popup_context(parent, thread, application);
 
 #if DEBUG_GUI_MOST
   fprintf(stderr, "show_popup: link=\"%s\", x_pos=%d, y_pos=%d\n",
