@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.c,v 1.11 2003/11/23 17:07:26 fuyu Exp $
+ * $Id: boardlist_view.c,v 1.15 2003/12/05 09:21:52 fuyu Exp $
  */
 
 #include "config.h"
@@ -43,6 +43,7 @@
 #include <stdio.h>
 #endif
 #include <stdlib.h>
+#include <string.h>
 
 
 static void boardlist_view_class_init(BoardlistViewClass *klass);
@@ -158,6 +159,8 @@ static gint boardlist_view_signals[LAST_SIGNAL] =
 static GtkItemFactory *popup_menu_item_factory = NULL;
 static GQuark node_info_id;
 
+static char *list_entry_color[4] = { NULL, NULL, NULL, NULL };
+
 
 #if TRACE_MEMORY_USAGE
 static void
@@ -191,27 +194,28 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 {
   GObjectClass *o_class = (GObjectClass *)klass;
   GtkObjectClass *object_class = (GtkObjectClass *)klass;
+  GtkWidgetClass *widget_class = (GtkWidgetClass *)klass;
   GtkBindingSet *binding_set;
 
   /* ݥåץåץ˥塼ν */
   GtkItemFactoryEntry menu_items[] =
     {
       {
-	_("/_Open Selection"),			/* menu path */
+	_("/_Open"),				/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_OPEN_SELECTION,		/* act */
 	NULL					/* type */
       },
       {
-	_("/Open Selected Boards in _Tab"),	/* menu path */
+	_("/Open Boards in _Tab"),		/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_OPEN_SELECTION_IN_TAB,	/* act */
 	NULL					/* type */
       },
       {
-	_("/Open Selected Boards with Web Browser"),	/* menu path */
+	_("/Open Boards with Web Browser"),	/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_OPEN_SELECTION_WITH_BROWSER,	/* act */
@@ -225,14 +229,14 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 	"<Separator>"
       },
       {
-	_("/Toggle Hide Selection(_D)"),	/* menu path */
+	_("/Toggle Hide(_D)"),			/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_TOGGLE_HIDE_SELECTION,	/* act */
 	NULL					/* type */
       },
       {
-	_("/Kill Selection(_K)"),		/* menu path */
+	_("/Kill(_K)"),				/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_KILL_SELECTION,		/* act */
@@ -256,6 +260,31 @@ boardlist_view_class_init(BoardlistViewClass *klass)
   /* GtkObject signals */
   object_class->destroy = boardlist_view_destroy;
 
+  gtk_widget_class_install_style_property(widget_class,
+					  g_param_spec_string("fg_normal",
+							      _("FG normal"),
+							      _("Foreground color for ordinal list entry"),
+							      NULL,
+							      G_PARAM_READABLE));
+  gtk_widget_class_install_style_property(widget_class,
+					  g_param_spec_string("fg_hidden",
+							      _("FG hidden"),
+							      _("Foreground color for hidden list entry"),
+							      "gray75",
+							      G_PARAM_READABLE));
+  gtk_widget_class_install_style_property(widget_class,
+					  g_param_spec_string("bg_normal",
+							      _("BG normal"),
+							      _("Background color for ordinal list entry"),
+							      NULL,
+							      G_PARAM_READABLE));
+  gtk_widget_class_install_style_property(widget_class,
+					  g_param_spec_string("bg_hidden",
+							      _("BG hidden"),
+							      _("Background color for hidden list entry"),
+							      NULL,
+							      G_PARAM_READABLE));
+
   boardlist_view_signals[OPEN_THREADLIST_VIEW_SIGNAL] =
     g_signal_new("open_threadlist_view",
 		 G_TYPE_FROM_CLASS(klass),
@@ -519,6 +548,23 @@ boardlist_view_destroy(GtkObject *object)
 }
 
 
+gboolean
+boardlist_view_get_default_open_in_tab(BoardlistView *view)
+{
+  g_return_val_if_fail(IS_BOARDLIST_VIEW(view), FALSE);
+  return view->default_open_in_tab;
+}
+
+
+void
+boardlist_view_set_default_open_in_tab(BoardlistView *view,
+				       gboolean default_open_in_tab)
+{
+  g_return_if_fail(IS_BOARDLIST_VIEW(view));
+  view->default_open_in_tab = default_open_in_tab;
+}
+
+
 void
 boardlist_view_expand_category(BoardlistView *view,
 			       OchushaBoardCategory *category)
@@ -752,6 +798,16 @@ boardlist_view_do_on_item_at_cursor(BoardlistView *view, gint what_to_do)
 	  break;
 
 	case BACK_VIEW:
+	  if (OCHUSHA_IS_BOARD_CATEGORY(data))
+	    {
+	      if (gtk_tree_view_row_expanded(&view->view, path))
+		boardlist_view_collapse_category(view,
+						 OCHUSHA_BOARD_CATEGORY(data));
+	      else
+		boardlist_view_expand_category(view,
+					       OCHUSHA_BOARD_CATEGORY(data));
+	      break;
+	    }
 	  g_signal_emit(G_OBJECT(view),
 			boardlist_view_signals[BACK_VIEW_SIGNAL],
 			0,
@@ -855,7 +911,8 @@ tree_view_button_press_event_cb(GtkWidget *widget, GdkEventButton *event,
 		    boardlist_view_signals[OPEN_THREADLIST_VIEW_SIGNAL],
 		    0,
 		    data,
-		    event->button == 2,
+		    view->default_open_in_tab
+		    ? event->button == 1 : event->button == 2,
 		    FALSE);
       break;
 
@@ -958,12 +1015,127 @@ tree_view_row_collapsed_cb(GtkTreeView *tree_view,
 }
 
 
+static const char *boardlist_view_get_color(BoardlistView *view,
+					    guint color_code);
+
+static const char *
+find_replacement_color(BoardlistView *view, const char *color)
+{
+  const char *result_color;
+  GdkColor *gdk_color;
+  char name_buf[8];
+  g_return_val_if_fail(color != NULL && *color == '<', NULL);
+
+  if (strncmp(color + 1, "fg_", 3) == 0)
+    {
+      if (strcmp(color + 4, "normal>") == 0)
+	result_color = boardlist_view_get_color(view, 0);
+      else if (strcmp(color + 4, "hidden>") == 0)
+	result_color = boardlist_view_get_color(view, 1);
+      else
+	return NULL;
+      if (result_color != NULL)
+	return result_color;
+
+      gdk_color = &GTK_WIDGET(view)->style->text[GTK_STATE_NORMAL];
+      snprintf(name_buf, 8, "#%02x%02x%02x",
+	       gdk_color->red / 256, gdk_color->green / 256,
+	       gdk_color->blue / 256);
+      return g_strdup(name_buf);	/* Ĥϥ꡼뤱̵ */
+    }
+  else if (strncmp(color + 1, "bg_", 3) == 0)
+    {
+      if (strcmp(color + 4, "normal>") == 0)
+	result_color = boardlist_view_get_color(view, 2);
+      else if (strcmp(color + 4, "hidden>") == 0)
+	result_color = boardlist_view_get_color(view, 3);
+      else
+	return NULL;
+      if (result_color != NULL)
+	return result_color;
+
+      gdk_color = &GTK_WIDGET(view)->style->base[GTK_STATE_NORMAL];
+      snprintf(name_buf, 8, "#%02x%02x%02x",
+	       gdk_color->red / 256, gdk_color->green / 256,
+	       gdk_color->blue / 256);
+      return g_strdup(name_buf);	/* Ĥϥ꡼뤱̵ */
+    }
+
+  return NULL;
+}
+
+
+static const char *
+boardlist_view_get_color(BoardlistView *view, guint color_code)
+{
+  const char *color;
+  char *tmp_color;
+
+  g_return_val_if_fail(color_code < 4, NULL);
+
+  if (list_entry_color[color_code] != NULL)
+    {
+      color = list_entry_color[color_code];
+      if (color != NULL && *color == '\0')
+	return NULL;
+      return color;
+    }
+
+  switch (color_code)
+    {
+    case 0:
+      gtk_widget_style_get(GTK_WIDGET(view), "fg_normal", &color, NULL);
+      break;
+    case 1:
+      gtk_widget_style_get(GTK_WIDGET(view), "fg_hidden", &color, NULL);
+      break;
+    case 2:
+      gtk_widget_style_get(GTK_WIDGET(view), "bg_normal", &color, NULL);
+      break;
+    case 3:
+      gtk_widget_style_get(GTK_WIDGET(view), "bg_hidden", &color, NULL);
+      break;
+    default:
+      color = NULL;
+    }
+
+  if (color == NULL)
+    return color;
+
+  if (*color == '\0')
+    return NULL;
+
+  if (*color != '<')
+    return color;
+
+  list_entry_color[color_code] = G_STRDUP("");
+
+  color = find_replacement_color(view, color);
+
+  if (color == NULL)
+    return NULL;
+
+  tmp_color = G_STRDUP(color);
+  G_FREE(list_entry_color[color_code]);
+
+  list_entry_color[color_code] = tmp_color;
+  if (*tmp_color == '\0')
+    return NULL;
+  return tmp_color;
+}
+
+
 static void
 decorate_boardlist_entry(GtkTreeViewColumn *tree_column,
 			 GtkCellRenderer *renderer, GtkTreeModel *tree_model,
 			 GtkTreeIter *iter, gpointer view)
 {
   guint state;
+  GValue st = { 0, };
+  GValue bg = { 0, };
+  GValue fg = { 0, };
+  const char *color;
+
   g_return_if_fail(gtk_tree_store_iter_is_valid(GTK_TREE_STORE(tree_model),
 						iter));
   gtk_tree_model_get(tree_model, iter,
@@ -974,37 +1146,57 @@ decorate_boardlist_entry(GtkTreeViewColumn *tree_column,
 
   if ((state & BOARDLIST_ROW_STATE_HIDDEN) != 0)
     {
-      GValue st = { 0, };
-      GValue fg = { 0, };
       g_value_init(&st, G_TYPE_BOOLEAN);
       g_value_set_boolean(&st, TRUE);
       g_object_set_property((GObject *)renderer, "strikethrough", &st);
       g_object_set_property((GObject *)renderer, "strikethrough_set", &st);
 
-      g_value_init(&fg, G_TYPE_STRING);
-      g_value_set_static_string(&fg, "gray75");
-      g_object_set_property((GObject *)renderer, "foreground", &fg);
+      color = boardlist_view_get_color(view, 3);
+      g_value_init(&bg, G_TYPE_STRING);
+      g_value_set_static_string(&bg, color);
+      g_object_set_property((GObject *)renderer, "cell_background", &bg);
+
+      if ((state & BOARDLIST_ROW_STATE_KILLED) != 0)
+	{
+	  color = boardlist_view_get_color(view, 3);
+	  g_value_init(&fg, G_TYPE_STRING);
+	  g_value_set_static_string(&fg, color);
+	  g_object_set_property((GObject *)renderer, "foreground", &fg);
+	}
+      else
+	{
+	  color = boardlist_view_get_color(view, 1);
+	  g_value_init(&fg, G_TYPE_STRING);
+	  g_value_set_static_string(&fg, color);
+	  g_object_set_property((GObject *)renderer, "foreground", &fg);
+	}
     }
   else
     {
-      GValue st = { 0, };
-      GValue fg = { 0, };
       g_value_init(&st, G_TYPE_BOOLEAN);
       g_value_set_boolean(&st, FALSE);
       g_object_set_property((GObject *)renderer, "strikethrough", &st);
       g_object_set_property((GObject *)renderer, "strikethrough_set", &st);
 
-      g_value_init(&fg, G_TYPE_STRING);
-      g_value_set_static_string(&fg, NULL);
-      g_object_set_property((GObject *)renderer, "foreground", &fg);
-    }
+      color = boardlist_view_get_color(view, 2);
+      g_value_init(&bg, G_TYPE_STRING);
+      g_value_set_static_string(&bg, color);
+      g_object_set_property((GObject *)renderer, "cell_background", &bg);
 
-  if ((state & BOARDLIST_ROW_STATE_KILLED) != 0)
-    {
-      GValue fg = { 0, };
-      g_value_init(&fg, G_TYPE_STRING);
-      g_value_set_static_string(&fg, "white");
-      g_object_set_property((GObject *)renderer, "foreground", &fg);
+      if ((state & BOARDLIST_ROW_STATE_KILLED) != 0)
+	{
+	  color = boardlist_view_get_color(view, 2);
+	  g_value_init(&fg, G_TYPE_STRING);
+	  g_value_set_static_string(&fg, color);
+	  g_object_set_property((GObject *)renderer, "foreground", &fg);
+	}
+      else
+	{
+	  color = boardlist_view_get_color(view, 0);
+	  g_value_init(&fg, G_TYPE_STRING);
+	  g_value_set_static_string(&fg, color);
+	  g_object_set_property((GObject *)renderer, "foreground", &fg);
+	}
     }
 }
 
