@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.68 2003/12/11 17:13:41 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.69 2003/12/12 21:43:52 fuyu Exp $
  */
 
 #include "config.h"
@@ -265,10 +265,32 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
   if ((info->view_flags & BBS_THREAD_HIDDEN) != 0)
     thread->number_of_responses_read = 0;
 
+  info->thread_local_a_bone
+    = ochusha_utils_get_attribute_boolean(hash_table, "thread_local_a_bone");
+  info->a_bone_by_name
+    = ochusha_utils_get_attribute_boolean(hash_table, "a_bone_by_name");
+  info->a_bone_by_name_pattern
+    = ochusha_utils_get_attribute_string(hash_table, "a_bone_by_name_pattern");
+  info->a_bone_by_id
+    = ochusha_utils_get_attribute_boolean(hash_table, "a_bone_by_id");
+  info->a_bone_by_id_pattern
+    = ochusha_utils_get_attribute_string(hash_table, "a_bone_by_id_pattern");
+  info->a_bone_by_content
+    = ochusha_utils_get_attribute_boolean(hash_table, "a_bone_by_content");
+  info->a_bone_by_content_pattern
+    = ochusha_utils_get_attribute_string(hash_table,
+					 "a_bone_by_content_pattern");
+
   return;
 }
 
 
+#define OUTPUT_THREAD_ATTRIBUTE_BOOLEAN(file, thread, attribute)	\
+  fprintf(file,								\
+	  "      <attribute name=\"" #attribute	"\">\n"			\
+	  "        <boolean val=\"%s\"/>\n"				\
+	  "      </attribute>\n", (thread)->attribute ? "true" : "false")
+
 #define OUTPUT_THREAD_ATTRIBUTE_INT(file, thread, attribute)		\
   fprintf(file,								\
 	  "      <attribute name=\"" #attribute	"\">\n"			\
@@ -315,6 +337,21 @@ threadlist_write_thread_element_cb(OchushaBulletinBoard *board,
       thread->number_of_responses_read = 0;
       ochusha_bbs_thread_remove_cache(thread, &application->config);
     }
+
+  if (info->thread_local_a_bone)
+    {
+      OUTPUT_THREAD_ATTRIBUTE_BOOLEAN(threadlist_xml, info,
+				      thread_local_a_bone);
+      OUTPUT_THREAD_ATTRIBUTE_BOOLEAN(threadlist_xml, info, a_bone_by_name);
+      OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, info,
+				     a_bone_by_name_pattern);
+      OUTPUT_THREAD_ATTRIBUTE_BOOLEAN(threadlist_xml, info, a_bone_by_id);
+      OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, info,
+				     a_bone_by_id_pattern);
+      OUTPUT_THREAD_ATTRIBUTE_BOOLEAN(threadlist_xml, info, a_bone_by_content);
+      OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, info,
+				     a_bone_by_content_pattern);
+    }
 }
 
 
@@ -342,6 +379,24 @@ bulletin_board_gui_info_free(BulletinBoardGUIInfo *info)
       info->last_mail = NULL;
     }
 
+  if (info->a_bone_by_name_pattern != NULL)
+    {
+      G_FREE(info->a_bone_by_name_pattern);
+      info->a_bone_by_name_pattern = NULL;
+    }
+
+  if (info->a_bone_by_id_pattern != NULL)
+    {
+      G_FREE(info->a_bone_by_id_pattern);
+      info->a_bone_by_id_pattern = NULL;
+    }
+
+  if (info->a_bone_by_content_pattern != NULL)
+    {
+      G_FREE(info->a_bone_by_content_pattern);
+      info->a_bone_by_content_pattern = NULL;
+    }
+
   G_FREE(info);
 }
 
@@ -441,7 +496,6 @@ title_match_func(const gchar *title, ThreadlistSearchQueryData *search_data)
 #endif
     }
 
-#if ENABLE_ONIGURUMA
   if (search_data->use_regexp)
     {
       if (search_data->regexp_available)
@@ -452,9 +506,6 @@ title_match_func(const gchar *title, ThreadlistSearchQueryData *search_data)
     }
   else
     result = (strstr(normalized_title, search_data->key) != NULL);
-#else
-  result = (strstr(normalized_title, search_data->key) != NULL);
-#endif
 
   G_FREE(normalized_title);
 
