@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.h,v 1.18 2003/12/07 16:57:58 fuyu Exp $
+ * $Id: bulletin_board_ui.h,v 1.19 2003/12/12 21:43:52 fuyu Exp $
  */
 
 #ifndef _BULLETIN_BOARD_UI_H_
@@ -41,12 +41,10 @@
 
 #include <gtk/gtk.h>
 
-#if ENABLE_ONIGURUMA
-# if HAVE_ONIG_ONIGPOSIX_H
-#  include <onig/onigposix.h>
-# else
-#  include <onigposix.h>
-# endif
+#if HAVE_ONIG_ONIGPOSIX_H
+# include <onig/onigposix.h>
+#else
+# include <onigposix.h>
 #endif
 
 #ifdef __cplusplus
@@ -71,9 +69,25 @@ typedef struct _BulletinBoardGUIInfo
   /* Ǹν񤭹߻Υᥤ */
   gchar *last_mail;
 
+  /* ܡ */
+  gboolean board_local_a_bone;
+
+  gboolean a_bone_by_name;
+  gchar *a_bone_by_name_pattern;
+
+  gboolean a_bone_by_id;
+  gchar *a_bone_by_id_pattern;
+
+  gboolean a_bone_by_content;
+  gchar *a_bone_by_content_pattern;
+
+
+  /* ե륿 */
   ThreadlistFilter filter;
 
   gboolean enable_filter;
+
+  /* Ʊ */
   volatile gboolean is_busy;
   volatile gboolean redraw_required;
 } BulletinBoardGUIInfo;
@@ -85,10 +99,8 @@ typedef struct _ThreadlistSearchQueryData
   gint direction;
   ThreadlistView *last_view;
 
-#if ENABLE_ONIGURUMA
   regex_t regexp;
   gboolean regexp_available;
-#endif
 
   gboolean enable_wrap;
   gboolean match_case;
