@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.h,v 1.27 2003/12/03 12:01:10 fuyu Exp $
+ * $Id: ochusha_ui.h,v 1.28 2003/12/12 21:43:52 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_H_
@@ -56,8 +56,6 @@
 #define ENABLE_MAIN_TOOLBAR						0
 #define ENABLE_RENDERING_ON_IDLE					0
 
-#define ENABLE_MULTIPLE_POPUPS						1
-
 #define OCHUSHA_DEFAULT_WIDTH						600
 #define OCHUSHA_DEFAULT_HEIGHT						400
 #define OCHUSHA_DEFAULT_BOARDLIST_WIDTH					150
@@ -208,16 +206,29 @@ struct _OchushaApplication
   gboolean thread_tab_always_show;
   gboolean show_threadlist_entry_when_thread_selected;
   char *threadlist_view_contents;
+  gboolean default_open_in_tab;
 
 
   /* ɽϢ */
   char *thread_view_font_name;
+  gboolean enable_transparent_a_bone;
+  gboolean disable_popup_a_bone;
+
 
   /* ΤΥǥե */
-  gboolean default_open_in_tab;
   ThreadViewMailtoMode show_mailto_mode;
 
 
+  gboolean a_bone_by_name;
+  gchar *a_bone_by_name_pattern;
+
+  gboolean a_bone_by_id;
+  gchar *a_bone_by_id_pattern;
+
+  gboolean a_bone_by_content;
+  gchar *a_bone_by_content_pattern;
+
+
   /* 쥿Υݥåץå״Ϣ */
   gboolean enable_popup_title;
   guint popup_title_delay;
