@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: paned_notebook.c,v 1.5 2003/11/26 10:05:26 fuyu Exp $
+ * $Id: paned_notebook.c,v 1.8 2003/12/05 08:41:26 fuyu Exp $
  */
 
 #include "config.h"
@@ -363,17 +363,11 @@ paned_notebook_init(PanedNotebook *paned_notebook)
   paned_notebook->toolbar_box = GTK_BOX(gtk_hbox_new(FALSE, 0));
 
   paned_notebook->toolbar = GTK_TOOLBAR(gtk_toolbar_new());
-  gtk_toolbar_set_style(paned_notebook->toolbar, GTK_TOOLBAR_ICONS);
-  gtk_toolbar_set_icon_size(paned_notebook->toolbar,
-			    GTK_ICON_SIZE_SMALL_TOOLBAR);
   gtk_box_pack_start(paned_notebook->toolbar_box,
 		     GTK_WIDGET(paned_notebook->toolbar),
 		     TRUE, TRUE, 0);
 
   paned_notebook->right_toolbar = GTK_TOOLBAR(gtk_toolbar_new());
-  gtk_toolbar_set_style(paned_notebook->right_toolbar, GTK_TOOLBAR_ICONS);
-  gtk_toolbar_set_icon_size(paned_notebook->right_toolbar,
-			    GTK_ICON_SIZE_SMALL_TOOLBAR);
   gtk_box_pack_end(paned_notebook->toolbar_box,
 		   GTK_WIDGET(paned_notebook->right_toolbar),
 		   FALSE, TRUE, 0);
@@ -418,10 +412,6 @@ paned_notebook_init(PanedNotebook *paned_notebook)
 
   paned_notebook->item_contents = MOD_NOTEBOOK(mod_notebook_new());
   mod_notebook_set_scrollable(paned_notebook->item_contents, TRUE);
-#if 0
-  mod_notebook_set_tab_shrinkable(paned_notebook->item_contents, TRUE);
-  mod_notebook_set_minimum_tab_label_size(paned_notebook->item_contents, 64);
-#endif
   mod_notebook_set_show_tabs(paned_notebook->item_contents, FALSE);
 
   g_signal_connect(G_OBJECT(paned_notebook->item_contents),
@@ -597,6 +587,8 @@ paned_notebook_set_tab_policy(PanedNotebook *paned_notebook,
     case GTK_POLICY_AUTOMATIC:
       if (paned_notebook->number_of_items_shown > 1)
 	mod_notebook_set_show_tabs(paned_notebook->item_contents, TRUE);
+      else
+	mod_notebook_set_show_tabs(paned_notebook->item_contents, FALSE);
       break;
 
     case GTK_POLICY_NEVER:
@@ -673,12 +665,15 @@ paned_notebook_set_always_show_contents_pane(PanedNotebook *paned_notebook,
   if (show_always)
     {
       gtk_widget_show_all(GTK_WIDGET(paned_notebook->contents_pane));
-      gtk_paned_add2(GTK_PANED(&paned_notebook->container),
+      gtk_paned_add2(GTK_PANED(paned_notebook),
 		     GTK_WIDGET(paned_notebook->contents_pane));
     }
   else
     {
-      gtk_container_remove(GTK_CONTAINER(&paned_notebook->container),
+      if (gtk_paned_get_position(GTK_PANED(paned_notebook)) == 0)
+	paned_notebook->position_zero_hack = TRUE;
+
+      gtk_container_remove(GTK_CONTAINER(paned_notebook),
 			   GTK_WIDGET(paned_notebook->contents_pane));
     }
 }
@@ -708,9 +703,9 @@ paned_notebook_size_request(GtkWidget *widget, GtkRequisition *requisition)
   requisition->width = 0;
   requisition->height = 0;
 
-  if (paned->child1 && GTK_WIDGET_VISIBLE (paned->child1))
+  if (paned->child1 && GTK_WIDGET_VISIBLE(paned->child1))
     {
-      gtk_widget_size_request (paned->child1, &child_requisition);
+      gtk_widget_size_request(paned->child1, &child_requisition);
 
       requisition->height = child_requisition.height;
       requisition->width = child_requisition.width;
@@ -718,7 +713,7 @@ paned_notebook_size_request(GtkWidget *widget, GtkRequisition *requisition)
 
   if (paned->child2 && GTK_WIDGET_VISIBLE (paned->child2))
     {
-      gtk_widget_size_request (paned->child2, &child_requisition);
+      gtk_widget_size_request(paned->child2, &child_requisition);
 
       if (paned_notebook->paned_style == HPANED_STYLE)
 	{
@@ -728,21 +723,21 @@ paned_notebook_size_request(GtkWidget *widget, GtkRequisition *requisition)
 	}
       else
 	{
-	  requisition->width = MAX (requisition->width,
-				    child_requisition.width);
+	  requisition->width = MAX(requisition->width,
+				   child_requisition.width);
 	  requisition->height += child_requisition.height;
 	}
     }
 
-  requisition->height += GTK_CONTAINER (paned)->border_width * 2;
-  requisition->width += GTK_CONTAINER (paned)->border_width * 2;
+  requisition->height += GTK_CONTAINER(paned)->border_width * 2;
+  requisition->width += GTK_CONTAINER(paned)->border_width * 2;
   
-  if (paned->child1 && GTK_WIDGET_VISIBLE (paned->child1) &&
-      paned->child2 && GTK_WIDGET_VISIBLE (paned->child2))
+  if (paned->child1 && GTK_WIDGET_VISIBLE(paned->child1) &&
+      paned->child2 && GTK_WIDGET_VISIBLE(paned->child2))
     {
       gint handle_size;
 
-      gtk_widget_style_get (widget, "handle_size", &handle_size, NULL);
+      gtk_widget_style_get(widget, "handle_size", &handle_size, NULL);
       if (paned_notebook->paned_style == HPANED_STYLE)
 	requisition->width += handle_size;
       else
@@ -752,7 +747,7 @@ paned_notebook_size_request(GtkWidget *widget, GtkRequisition *requisition)
 
 
 static void
-paned_notebook_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
+paned_notebook_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
   PanedNotebook *paned_notebook = PANED_NOTEBOOK(widget);
   GtkPaned *paned = GTK_PANED (widget);
@@ -760,8 +755,8 @@ paned_notebook_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 
   widget->allocation = *allocation;
 
-  if (paned->child1 && GTK_WIDGET_VISIBLE (paned->child1) &&
-      paned->child2 && GTK_WIDGET_VISIBLE (paned->child2))
+  if (paned->child1 && GTK_WIDGET_VISIBLE(paned->child1) &&
+      paned->child2 && GTK_WIDGET_VISIBLE(paned->child2))
     {
       GtkAllocation child1_allocation;
       GtkAllocation child2_allocation;
@@ -771,81 +766,81 @@ paned_notebook_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
       
       gtk_widget_style_get (widget, "handle_size", &handle_size, NULL);
 
-      gtk_widget_get_child_requisition (paned->child1, &child1_requisition);
-      gtk_widget_get_child_requisition (paned->child2, &child2_requisition);
+      gtk_widget_get_child_requisition(paned->child1, &child1_requisition);
+      gtk_widget_get_child_requisition(paned->child2, &child2_requisition);
 
       if (paned_notebook->paned_style == HPANED_STYLE)
 	{
-	  gtk_paned_compute_position (paned,
-				      MAX (1, widget->allocation.width
-					   - handle_size
-					   - 2 * border_width),
-				      child1_requisition.width,
-				      child2_requisition.width);
+	  gtk_paned_compute_position(paned,
+				     MAX(1, widget->allocation.width
+					 - handle_size
+					 - 2 * border_width),
+				     child1_requisition.width,
+				     child2_requisition.width);
 
 	  paned->handle_pos.x = widget->allocation.x + paned->child1_size + border_width;
 	  paned->handle_pos.y = widget->allocation.y + border_width;
 	  paned->handle_pos.width = handle_size;
-	  paned->handle_pos.height = MAX (1, widget->allocation.height - 2 * border_width);
+	  paned->handle_pos.height = MAX(1, widget->allocation.height - 2 * border_width);
 	}
       else
 	{
-	  gtk_paned_compute_position (paned,
-				      MAX (1, widget->allocation.height
-					   - handle_size
-					   - 2 * border_width),
-				      child1_requisition.height,
-				      child2_requisition.height);
+	  gtk_paned_compute_position(paned,
+				     MAX(1, widget->allocation.height
+					 - handle_size
+					 - 2 * border_width),
+				     child1_requisition.height,
+				     child2_requisition.height);
 
 	  paned->handle_pos.x = widget->allocation.x + border_width;
 	  paned->handle_pos.y = widget->allocation.y + paned->child1_size + border_width;
-	  paned->handle_pos.width = MAX (1, (gint) widget->allocation.width - 2 * border_width);
+	  paned->handle_pos.width = MAX(1, (gint) widget->allocation.width - 2 * border_width);
 	  paned->handle_pos.height = handle_size;
 	}
       
       
-      if (GTK_WIDGET_REALIZED (widget))
+      if (GTK_WIDGET_REALIZED(widget))
 	{
-	  if (GTK_WIDGET_MAPPED (widget))
-	    gdk_window_show (paned->handle);
+	  if (GTK_WIDGET_MAPPED(widget))
+	    gdk_window_show(paned->handle);
 	  if (paned_notebook->paned_style == HPANED_STYLE)
 	    {
-	      gdk_window_move_resize (paned->handle,
-				      paned->handle_pos.x,
-				      paned->handle_pos.y,
-				      handle_size,
-				      paned->handle_pos.height);
+	      gdk_window_move_resize(paned->handle,
+				     paned->handle_pos.x,
+				     paned->handle_pos.y,
+				     handle_size,
+				     paned->handle_pos.height);
 	    }
 	  else
 	    {
-	      gdk_window_move_resize (paned->handle,
-				      paned->handle_pos.x,
-				      paned->handle_pos.y,
-				      paned->handle_pos.width,
-				      handle_size);
+	      gdk_window_move_resize(paned->handle,
+				     paned->handle_pos.x,
+				     paned->handle_pos.y,
+				     paned->handle_pos.width,
+				     handle_size);
 	    }
 	}
 
       if (paned_notebook->paned_style == HPANED_STYLE)
 	{
-	  child1_allocation.height = child2_allocation.height = MAX (1, (gint) allocation->height - border_width * 2);
+	  child1_allocation.height = child2_allocation.height = MAX(1, (gint) allocation->height - border_width * 2);
 	  child1_allocation.width = paned->child1_size;
 	  child1_allocation.x = widget->allocation.x + border_width;
 	  child1_allocation.y = child2_allocation.y = widget->allocation.y + border_width;
 
 	  child2_allocation.x = child1_allocation.x + child1_allocation.width +  paned->handle_pos.width;
-	  child2_allocation.width = MAX (1, widget->allocation.x + widget->allocation.width - child2_allocation.x - border_width);
+	  child2_allocation.width = MAX(1, widget->allocation.x + widget->allocation.width - child2_allocation.x - border_width);
 
 	}
       else
 	{
-	  child1_allocation.width = child2_allocation.width = MAX (1, (gint) allocation->width - border_width * 2);
+	  child1_allocation.width = child2_allocation.width = MAX(1, (gint) allocation->width - border_width * 2);
 	  child1_allocation.height = paned->child1_size;
 	  child1_allocation.x = child2_allocation.x = widget->allocation.x + border_width;
 	  child1_allocation.y = widget->allocation.y + border_width;
 
 	  child2_allocation.y = child1_allocation.y + child1_allocation.height + paned->handle_pos.height;
-	  child2_allocation.height = MAX (1, widget->allocation.y + widget->allocation.height - child2_allocation.y - border_width);
+	  child2_allocation.height = MAX(1, widget->allocation.y + widget->allocation.height - child2_allocation.y - border_width);
 	}
       
       
@@ -854,30 +849,30 @@ paned_notebook_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
        */
       if (paned_notebook->paned_style == HPANED_STYLE)
 	{
-	  if (GTK_WIDGET_MAPPED (widget) &&
+	  if (GTK_WIDGET_MAPPED(widget) &&
 	      paned->child1->allocation.width < child1_allocation.width)
 	    {
-	      gtk_widget_size_allocate (paned->child2, &child2_allocation);
-	      gtk_widget_size_allocate (paned->child1, &child1_allocation);
+	      gtk_widget_size_allocate(paned->child2, &child2_allocation);
+	      gtk_widget_size_allocate(paned->child1, &child1_allocation);
 	    }
 	  else
 	    {
-	      gtk_widget_size_allocate (paned->child1, &child1_allocation);
-	      gtk_widget_size_allocate (paned->child2, &child2_allocation);
+	      gtk_widget_size_allocate(paned->child1, &child1_allocation);
+	      gtk_widget_size_allocate(paned->child2, &child2_allocation);
 	    }
 	}
       else
 	{
-	  if (GTK_WIDGET_MAPPED (widget) &&
+	  if (GTK_WIDGET_MAPPED(widget) &&
 	      paned->child1->allocation.height < child1_allocation.height)
 	    {
-	      gtk_widget_size_allocate (paned->child2, &child2_allocation);
-	      gtk_widget_size_allocate (paned->child1, &child1_allocation);
+	      gtk_widget_size_allocate(paned->child2, &child2_allocation);
+	      gtk_widget_size_allocate(paned->child1, &child1_allocation);
 	    }
 	  else
 	    {
-	      gtk_widget_size_allocate (paned->child1, &child1_allocation);
-	      gtk_widget_size_allocate (paned->child2, &child2_allocation);
+	      gtk_widget_size_allocate(paned->child1, &child1_allocation);
+	      gtk_widget_size_allocate(paned->child2, &child2_allocation);
 	    }
 	}
     }
@@ -886,17 +881,29 @@ paned_notebook_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
       GtkAllocation child_allocation;
 
       if (GTK_WIDGET_REALIZED (widget))      
-	gdk_window_hide (paned->handle);
+	gdk_window_hide(paned->handle);
 	  
       child_allocation.x = widget->allocation.x + border_width;
       child_allocation.y = widget->allocation.y + border_width;
-      child_allocation.width = MAX (1, allocation->width - 2 * border_width);
-      child_allocation.height = MAX (1, allocation->height - 2 * border_width);
-      
-      if (paned->child1 && GTK_WIDGET_VISIBLE (paned->child1))
-	gtk_widget_size_allocate (paned->child1, &child_allocation);
-      else if (paned->child2 && GTK_WIDGET_VISIBLE (paned->child2))
-	gtk_widget_size_allocate (paned->child2, &child_allocation);
+      child_allocation.width = MAX(1, allocation->width - 2 * border_width);
+      child_allocation.height = MAX(1, allocation->height - 2 * border_width);
+
+      if (paned->child1 && GTK_WIDGET_VISIBLE(paned->child1))
+	{
+	  gtk_widget_size_allocate(paned->child1, &child_allocation);
+	  if (paned_notebook->position_zero_hack)
+	    {
+#if 0
+	      fprintf(stderr, "Checkpoint #1\n");
+#endif
+	      gtk_widget_unmap(paned->child1);
+	      gtk_widget_set_child_visible(paned->child1, TRUE);
+	      gtk_widget_map(paned->child1);
+	      paned_notebook->position_zero_hack = FALSE;
+	    }
+	}
+      else if (paned->child2 && GTK_WIDGET_VISIBLE(paned->child2))
+	gtk_widget_size_allocate(paned->child2, &child_allocation);
     }
 }
 
@@ -942,12 +949,12 @@ paned_notebook_set_selector(PanedNotebook *paned_notebook, GtkWidget *selector)
     return;	/* ⤷ʤ */
 
   if (paned_notebook->selector != NULL)
-    gtk_container_remove(GTK_CONTAINER(&paned_notebook->container),
+    gtk_container_remove(GTK_CONTAINER(paned_notebook),
 			 paned_notebook->selector);
 
   if (selector != NULL)
     {
-      gtk_paned_add1(GTK_PANED(&paned_notebook->container), selector);
+      gtk_paned_add1(GTK_PANED(paned_notebook), selector);
     }
 
   paned_notebook->selector = selector;
@@ -1170,9 +1177,9 @@ paned_notebook_open_item_view(PanedNotebook *paned_notebook, gpointer item,
   if (paned_notebook->number_of_items_shown == 0
       && !paned_notebook->always_show_contents_pane)
     {
-      gtk_paned_add2(GTK_PANED(&paned_notebook->container),
-		     GTK_WIDGET(paned_notebook->contents_pane));
       gtk_widget_show_all(GTK_WIDGET(paned_notebook->contents_pane));
+      gtk_paned_add2(GTK_PANED(paned_notebook),
+		     GTK_WIDGET(paned_notebook->contents_pane));
     }
 
   if (paned_notebook->number_of_items_shown++ == 0 || in_tab)
@@ -1290,8 +1297,13 @@ close_nth_page(PanedNotebook *paned_notebook, gint page)
 
   if (paned_notebook->number_of_items_shown == 0
       && !paned_notebook->always_show_contents_pane)
-    gtk_container_remove(GTK_CONTAINER(&paned_notebook->container),
-			 GTK_WIDGET(paned_notebook->contents_pane));
+    {
+      if (gtk_paned_get_position(GTK_PANED(paned_notebook)) == 0)
+	paned_notebook->position_zero_hack = TRUE;
+
+      gtk_container_remove(GTK_CONTAINER(paned_notebook),
+			   GTK_WIDGET(paned_notebook->contents_pane));
+    }
 
   if (paned_notebook->number_of_items_shown == 1
       && paned_notebook->tab_policy == GTK_POLICY_AUTOMATIC)
