@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_view.c,v 1.22 2003/11/26 10:04:16 fuyu Exp $
+ * $Id: threadlist_view.c,v 1.30 2003/12/05 12:42:33 fuyu Exp $
  */
 
 #include "config.h"
@@ -94,7 +94,11 @@ static gboolean tree_view_button_release_event_cb(GtkWidget *widget,
 static void decorate_threadlist_entry(GtkTreeViewColumn *tree_column,
 				      GtkCellRenderer *renderer,
 				      GtkTreeModel *tree_model,
-				      GtkTreeIter *iter, gpointer unused);
+				      GtkTreeIter *iter, gpointer view);
+static void text_set_cell_data_rank(GtkTreeViewColumn *tree_column,
+				    GtkCellRenderer *renderer,
+				    GtkTreeModel *tree_model,
+				    GtkTreeIter *iter, gpointer view);
 static void active_set_cell_data_flags(GtkTreeViewColumn *tree_column,
 				       GtkCellRenderer *renderer,
 				       GtkTreeModel *tree_model,
@@ -153,6 +157,9 @@ enum {
   COPY_THREAD_URL_SIGNAL,
   ADVANCE_VIEW_SIGNAL,
   BACK_VIEW_SIGNAL,
+  WRITE_RESPONSE_SIGNAL,
+
+  CLOSE_THREADLIST_SIGNAL,
 
   INTERACTIVE_SEARCH_SIGNAL,
 
@@ -169,6 +176,7 @@ enum {
   TOGGLE_HIDDEN_STATE,
   ADVANCE_VIEW,
   BACK_VIEW,
+  WRITE_RESPONSE,
 };
 
 
@@ -193,51 +201,67 @@ struct _ThreadlistInfo
 
 static GtkTreeViewClass *parent_class = NULL;
 static gint threadlist_view_signals[LAST_SIGNAL] =
-  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 static GtkItemFactory *popup_menu_item_factory = NULL;
 static GQuark list_entry_info_id;
 
 
-static const char *list_entry_foreground_color[16] =
-{
-  NULL,	/* åȤdefault */
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-};
+static const char * const
+default_foreground_color[NUMBER_OF_LIST_ENTRY_FOREGROUND_COLORS] =
+  { NULL, "red", "red", "gray75" };
 
 
-static const char *list_entry_background_color[16] =
+static const char * const
+default_background_color[NUMBER_OF_LIST_ENTRY_BACKGROUND_COLORS] =
+  { NULL, "lavender", "#ffe0e0", NULL };
+
+
+static char *
+list_entry_foreground_color[NUMBER_OF_LIST_ENTRY_FOREGROUND_COLORS] =
+  { NULL, NULL, NULL, NULL };
+
+
+static char *
+list_entry_background_color[NUMBER_OF_LIST_ENTRY_BACKGROUND_COLORS] =
+  { NULL, NULL, NULL, NULL };
+
+
+void
+threadlist_entry_set_foreground_color(ThreadlistEntryForeground color,
+				      const char *color_name)
 {
-  NULL,	/* åȤdefault */
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-};
+  g_return_if_fail(color < NUMBER_OF_LIST_ENTRY_FOREGROUND_COLORS);
+
+  if (color_name == NULL)
+    color_name = default_foreground_color[color];
+
+  if (list_entry_foreground_color[color] != NULL)
+    G_FREE(list_entry_foreground_color[color]);
+
+  if (color_name == NULL)
+    list_entry_foreground_color[color] = NULL;
+  else
+    list_entry_foreground_color[color] = G_STRDUP(color_name);
+}
+
+
+void
+threadlist_entry_set_background_color(ThreadlistEntryBackground color,
+				      const char *color_name)
+{
+  g_return_if_fail(color < NUMBER_OF_LIST_ENTRY_BACKGROUND_COLORS);
+
+  if (color_name == NULL)
+    color_name = default_background_color[color];
+
+  if (list_entry_background_color[color] != NULL)
+    G_FREE(list_entry_background_color[color]);
+
+  if (color_name == NULL)
+    list_entry_background_color[color] = NULL;
+  else
+    list_entry_background_color[color] = G_STRDUP(color_name);
+}
 
 
 #ifndef HAVE_LOCALTIME_R
@@ -265,6 +289,7 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 {
   GObjectClass *o_class = (GObjectClass *)klass;
   GtkObjectClass *object_class = (GtkObjectClass *)klass;
+  GtkWidgetClass *widget_class = (GtkWidgetClass *)klass;
   GtkTreeViewClass *tree_view_class = (GtkTreeViewClass *)klass;
   GtkBindingSet *binding_set;
 
@@ -272,21 +297,21 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
   GtkItemFactoryEntry menu_items[] =
     {
       {
-	_("/_Open Selected Threads"),		/* menu path */
+	_("/_Open"),				/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_OPEN_SELECTION,		/* act */
 	NULL					/* type */
       },
       {
-	_("/Open Selected Threads in _Tab"),	/* menu path */
+	_("/Open in _Tab"),			/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_OPEN_SELECTION_IN_TAB,	/* act */
 	NULL					/* type */
       },
       {
-	_("/Open Selected Threads with Web Broser"),/* menu path */
+	_("/Open with _Web Browser"),		/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_OPEN_SELECTION_WITH_BROWSER,	/* act */
@@ -300,21 +325,21 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 	"<Separator>"
       },
       {
-	_("/Mark Selected Threads(_*)"),	/* menu path */
+	_("/Mark(_*)"),				/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_MARK_SELECTION,		/* act */
 	NULL					/* type */
       },
       {
-	_("/_Unmark Selected Threads"),		/* menu path */
+	_("/_Unmark"),				/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_UNMARK_SELECTION,		/* act */
 	NULL					/* type */
       },
       {
-	_("/Toggle Hide Selected Threads(_D)"),	/* menu path */
+	_("/Toggle Hide(_D)"),			/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_TOGGLE_HIDE_SELECTION,	/* act */
@@ -328,7 +353,7 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 	"<Separator>"
       },
       {
-	_("/Reset Selected Threads(_R)"),	/* menu path */
+	_("/Reset(_R)"),			/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_RESET_SELECTION,		/* act */
@@ -342,7 +367,7 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 	"<Separator>"
       },
       {
-	_("/Copy Selected Thread's URL"),	/* menu path */
+	_("/Copy Thread's URL"),		/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_COPY_THREAD_URL,		/* act */
@@ -367,6 +392,56 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
   /* GtkObject signals */
   object_class->destroy = threadlist_view_destroy;
 
+  gtk_widget_class_install_style_property(widget_class,
+					  g_param_spec_string("fg_normal",
+							      _("FG normal"),
+							      _("Foreground color for ordinal list entry"),
+							      NULL,
+							      G_PARAM_READABLE));
+  gtk_widget_class_install_style_property(widget_class,
+					  g_param_spec_string("fg_emph",
+							      _("FG emph"),
+							      _("Foreground color for list entry to be emphasized"),
+							      "red",
+							      G_PARAM_READABLE));
+  gtk_widget_class_install_style_property(widget_class,
+					  g_param_spec_string("fg_strong",
+							      _("FG strong"),
+							      _("Foreground color for list entry to be shown strongly"),
+							      NULL,
+							      G_PARAM_READABLE));
+  gtk_widget_class_install_style_property(widget_class,
+					  g_param_spec_string("fg_hidden",
+							      _("FG hidden"),
+							      _("Foreground color for hidden list entry"),
+							      "gray75",
+							      G_PARAM_READABLE));
+  gtk_widget_class_install_style_property(widget_class,
+					  g_param_spec_string("bg_normal",
+							      _("BG normal"),
+							      _("Background color for ordinal list entry"),
+							      NULL,
+							      G_PARAM_READABLE));
+  gtk_widget_class_install_style_property(widget_class,
+					  g_param_spec_string("bg_emph",
+							      _("BG emph"),
+							      _("Boreground color for list entry to be emphasized"),
+							      "lavender",
+							      G_PARAM_READABLE));
+  gtk_widget_class_install_style_property(widget_class,
+					  g_param_spec_string("bg_strong",
+							      _("BG strong"),
+							      _("Background color for list entry to be shown strongly"),
+							      "#ffe0e0",
+							      G_PARAM_READABLE));
+  gtk_widget_class_install_style_property(widget_class,
+					  g_param_spec_string("bg_hidden",
+							      _("BG hidden"),
+							      _("Background color for hidden list entry"),
+							      NULL,
+							      G_PARAM_READABLE));
+
+
   /* GtkTreeView signals */
   tree_view_class->move_cursor = threadlist_view_move_cursor;
 
@@ -466,6 +541,24 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 		 ochusha_marshal_BOOLEAN__OBJECT,
 		 G_TYPE_BOOLEAN, 1,
 		 OCHUSHA_TYPE_BBS_THREAD);
+  threadlist_view_signals[WRITE_RESPONSE_SIGNAL] =
+    g_signal_new("write_response",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(ThreadlistViewClass, write_response),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__OBJECT,
+		 G_TYPE_NONE, 1,
+		 OCHUSHA_TYPE_BBS_THREAD);
+
+  threadlist_view_signals[CLOSE_THREADLIST_SIGNAL] =
+    g_signal_new("close_threadlist",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		 G_STRUCT_OFFSET(ThreadlistViewClass, close_threadlist),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__VOID,
+		 G_TYPE_NONE, 0);
 
   threadlist_view_signals[INTERACTIVE_SEARCH_SIGNAL] =
     g_signal_new("interactive_search",
@@ -519,6 +612,9 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
   gtk_binding_entry_add_signal(binding_set, GDK_BackSpace, 0,
 			       "do_on_thread_at_cursor", 1,
 			       G_TYPE_INT, BACK_VIEW);
+  gtk_binding_entry_add_signal(binding_set, GDK_a, 0,
+			       "do_on_thread_at_cursor", 1,
+			       G_TYPE_INT, WRITE_RESPONSE);
 
   gtk_binding_entry_add_signal(binding_set, GDK_Up, 0,
 			       "move_cursor", 2,
@@ -572,6 +668,9 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 			       G_TYPE_INT,
 			       THREADLIST_SEARCH_ACTION_REGEXP_BACKWARD);
 
+  gtk_binding_entry_add_signal(binding_set, GDK_W, GDK_CONTROL_MASK,
+			       "close_threadlist", 0);
+
   gtk_binding_entry_add_signal(binding_set, GDK_Escape, 0,
 			       "interactive_search", 1,
 			       G_TYPE_INT,
@@ -591,6 +690,10 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
   klass->mark_thread = NULL;
   klass->advance_view = NULL;
   klass->back_view = NULL;
+  klass->write_response = NULL;
+
+  klass->close_threadlist = NULL;
+  klass->interactive_search = NULL;
 
   klass->do_on_selection = threadlist_view_do_on_selection;
   klass->do_on_thread_at_cursor = threadlist_view_do_on_thread_at_cursor;
@@ -605,22 +708,6 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
       abort();
     }
 #endif
-
-  /*
-   * XXX: ʲñʤǥեȤȤ
-   *      桼ˤ񤭴ǽˤ٤
-   */
-  list_entry_foreground_color[1] = g_strdup("blue");
-  list_entry_foreground_color[2] = g_strdup("red");
-  list_entry_foreground_color[3] = g_strdup("green");
-  list_entry_foreground_color[4] = g_strdup("gray50");
-  list_entry_foreground_color[5] = g_strdup("gray75");
-
-  list_entry_background_color[1] = g_strdup("lavender");
-  list_entry_background_color[2] = g_strdup("#ffe0e0");
-  list_entry_background_color[3] = g_strdup("DarkSeaGreen1");
-  list_entry_background_color[4] = g_strdup("gray75");
-  list_entry_background_color[5] = g_strdup("gray90");
 }
 
 
@@ -642,11 +729,6 @@ typedef enum
 static void
 threadlist_view_init(ThreadlistView *view)
 {
-#if 0
-  GtkTreeViewColumn *column;
-  GtkCellRenderer *renderer;
-#endif
-
   GTK_WIDGET_SET_FLAGS(view, GTK_CAN_FOCUS | GTK_RECEIVES_DEFAULT);
 
   view->selection = gtk_tree_view_get_selection(&view->view);
@@ -1260,6 +1342,13 @@ threadlist_view_do_on_thread_at_cursor(ThreadlistView *view, gint what_to_do)
 			data,
 			&result);
 	  break;
+
+	case WRITE_RESPONSE:
+	  g_signal_emit(G_OBJECT(view),
+			threadlist_view_signals[WRITE_RESPONSE_SIGNAL],
+			0,
+			data);
+	  break;
 	}
     }
 
@@ -1297,6 +1386,23 @@ threadlist_view_new(void)
 }
 
 
+gboolean
+threadlist_view_get_default_open_in_tab(ThreadlistView *view)
+{
+  g_return_val_if_fail(IS_THREADLIST_VIEW(view), FALSE);
+  return view->default_open_in_tab;
+}
+
+
+void
+threadlist_view_set_default_open_in_tab(ThreadlistView *view,
+					gboolean default_open_in_tab)
+{
+  g_return_if_fail(IS_THREADLIST_VIEW(view));
+  view->default_open_in_tab = default_open_in_tab;
+}
+
+
 static gboolean
 tree_view_motion_notify_event_cb(GtkWidget *widget, GdkEventMotion *event,
 				 ThreadlistView *view)
@@ -1406,7 +1512,8 @@ tree_view_button_press_event_cb(GtkWidget *widget, GdkEventButton *event,
 			    threadlist_view_signals[OPEN_THREAD_VIEW_SIGNAL],
 			    0,
 			    data,
-			    event->button == 2,
+			    view->default_open_in_tab
+			    ? event->button == 1 : event->button == 2,
 			    FALSE);
 	    }
 	}
@@ -1515,21 +1622,211 @@ threadlist_view_decode_visual_param_flags(ThreadlistView *view,
 }
 
 
-static guint
-threadlist_view_decode_visual_param_foreground_color(ThreadlistView *view,
-						     guint parameter)
+static const char *threadlist_view_decode_visual_param_foreground(ThreadlistView *view, guint parameter);
+static const char *threadlist_view_decode_visual_param_background(ThreadlistView *view, guint parameter);
+
+
+static const char *
+find_replacement_color(ThreadlistView *view, const char *color)
 {
-  return (parameter >> THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH) & 0xf;
+  const char *result_color;
+  GdkColor *gdk_color;
+  char name_buf[8];
+  g_return_val_if_fail(color != NULL && *color == '<', NULL);
+
+  if (strncmp(color + 1, "fg_", 3) == 0)
+    {
+      if (strcmp(color + 4, "normal>") == 0)
+	result_color = threadlist_view_decode_visual_param_foreground(view,
+				(LIST_ENTRY_FOREGROUND_NORMAL
+				 << THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH));
+      else if (strcmp(color + 4, "emph>") == 0)
+	result_color = threadlist_view_decode_visual_param_foreground(view,
+				(LIST_ENTRY_FOREGROUND_EMPH
+				 << THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH));
+      else if (strcmp(color + 4, "strong>") == 0)
+	result_color = threadlist_view_decode_visual_param_foreground(view,
+				(LIST_ENTRY_FOREGROUND_STRONG
+				 << THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH));
+      else if (strcmp(color + 4, "hidden>") == 0)
+	result_color = threadlist_view_decode_visual_param_foreground(view,
+				(LIST_ENTRY_FOREGROUND_HIDDEN
+				 << THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH));
+      else
+	return NULL;
+      if (result_color != NULL)
+	return result_color;
+
+      gdk_color = &GTK_WIDGET(view)->style->text[GTK_STATE_NORMAL];
+      snprintf(name_buf, 8, "#%02x%02x%02x",
+	       gdk_color->red / 256, gdk_color->green / 256,
+	       gdk_color->blue / 256);
+      return g_strdup(name_buf);	/* Ĥϥ꡼뤱̵ */
+    }
+  else if (strncmp(color + 1, "bg_", 3) == 0)
+    {
+      if (strcmp(color + 4, "normal>") == 0)
+	result_color = threadlist_view_decode_visual_param_background(view,
+		(LIST_ENTRY_BACKGROUND_NORMAL
+		 << (THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH
+		     + THREADLIST_ENTRY_VISUAL_FOREGROUND_COLOR_WIDTH)));
+      else if (strcmp(color + 4, "emph>") == 0)
+	result_color = threadlist_view_decode_visual_param_background(view,
+		(LIST_ENTRY_BACKGROUND_EMPH
+		 << (THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH
+		     + THREADLIST_ENTRY_VISUAL_FOREGROUND_COLOR_WIDTH)));
+      else if (strcmp(color + 4, "strong>") == 0)
+	result_color = threadlist_view_decode_visual_param_background(view,
+		(LIST_ENTRY_BACKGROUND_STRONG
+		 << (THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH
+		     + THREADLIST_ENTRY_VISUAL_FOREGROUND_COLOR_WIDTH)));
+      else if (strcmp(color + 4, "hidden>") == 0)
+	result_color = threadlist_view_decode_visual_param_background(view,
+		(LIST_ENTRY_BACKGROUND_HIDDEN
+		 << (THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH
+		     + THREADLIST_ENTRY_VISUAL_FOREGROUND_COLOR_WIDTH)));
+      else
+	return NULL;
+      if (result_color != NULL)
+	return result_color;
+
+      gdk_color = &GTK_WIDGET(view)->style->base[GTK_STATE_NORMAL];
+      snprintf(name_buf, 8, "#%02x%02x%02x",
+	       gdk_color->red / 256, gdk_color->green / 256,
+	       gdk_color->blue / 256);
+      return g_strdup(name_buf);	/* Ĥϥ꡼뤱̵ */
+    }
+
+  return NULL;
 }
 
 
-static guint
-threadlist_view_decode_visual_param_background_color(ThreadlistView *view,
-						     guint parameter)
+static const char *
+threadlist_view_decode_visual_param_foreground(ThreadlistView *view,
+					       guint parameter)
 {
-  return (parameter
-	  >> (THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH
-	      + THREADLIST_ENTRY_VISUAL_FOREGROUND_COLOR_WIDTH)) & 0xf;
+  guint color_code = (parameter >> THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH) & 0xf;
+  const char *color;
+  char *tmp_color;
+
+  g_return_val_if_fail(color_code < NUMBER_OF_LIST_ENTRY_FOREGROUND_COLORS,
+		       NULL);
+
+  if (list_entry_foreground_color[color_code] != NULL)
+    {
+      color = list_entry_foreground_color[color_code];
+      if (color != NULL && *color == '\0')
+	return NULL;
+      return color;
+    }
+
+  switch (color_code)
+    {
+    case LIST_ENTRY_FOREGROUND_NORMAL:
+      gtk_widget_style_get(GTK_WIDGET(view), "fg_normal", &color, NULL);
+      break;
+    case LIST_ENTRY_FOREGROUND_EMPH:
+      gtk_widget_style_get(GTK_WIDGET(view), "fg_emph", &color, NULL);
+      break;
+    case LIST_ENTRY_FOREGROUND_STRONG:
+      gtk_widget_style_get(GTK_WIDGET(view), "fg_strong", &color, NULL);
+      break;
+    case LIST_ENTRY_FOREGROUND_HIDDEN:
+      gtk_widget_style_get(GTK_WIDGET(view), "fg_hidden", &color, NULL);
+      break;
+    default:
+      color = NULL;
+    }
+
+  if (color == NULL)
+    return color;
+
+  if (*color == '\0')
+    return NULL;
+
+  if (*color != '<')
+    return color;
+
+  list_entry_foreground_color[color_code] = G_STRDUP("");
+
+  color = find_replacement_color(view, color);
+
+  if (color == NULL)
+    return NULL;
+
+  tmp_color = G_STRDUP(color);
+  G_FREE(list_entry_foreground_color[color_code]);
+
+  list_entry_foreground_color[color_code] = tmp_color;
+  if (*tmp_color == '\0')
+    return NULL;
+  return tmp_color;
+}
+
+
+static const char *
+threadlist_view_decode_visual_param_background(ThreadlistView *view,
+					       guint parameter)
+{
+  guint color_code
+    = (parameter
+       >> (THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH
+	   + THREADLIST_ENTRY_VISUAL_FOREGROUND_COLOR_WIDTH)) & 0xf;
+  const char *color;
+  char *tmp_color;
+
+  g_return_val_if_fail(color_code < NUMBER_OF_LIST_ENTRY_BACKGROUND_COLORS,
+		       NULL);
+
+  if (list_entry_background_color[color_code] != NULL)
+    {
+      color = list_entry_background_color[color_code];
+      if (color != NULL && *color == '\0')
+	return NULL;
+      return color;
+    }
+
+  switch (color_code)
+    {
+    case LIST_ENTRY_BACKGROUND_NORMAL:
+      gtk_widget_style_get(GTK_WIDGET(view), "bg_normal", &color, NULL);
+      break;
+    case LIST_ENTRY_BACKGROUND_EMPH:
+      gtk_widget_style_get(GTK_WIDGET(view), "bg_emph", &color, NULL);
+      break;
+    case LIST_ENTRY_BACKGROUND_STRONG:
+      gtk_widget_style_get(GTK_WIDGET(view), "bg_strong", &color, NULL);
+      break;
+    case LIST_ENTRY_BACKGROUND_HIDDEN:
+      gtk_widget_style_get(GTK_WIDGET(view), "bg_hidden", &color, NULL);
+      break;
+    default:
+      color = NULL;
+    }
+
+  if (color == NULL)
+    return color;
+
+  if (*color == '\0')
+    return NULL;
+
+  if (*color != '<')
+    return color;
+
+  list_entry_background_color[color_code] = G_STRDUP("");
+
+  color = find_replacement_color(view, color);
+
+  if (color == NULL)
+    return NULL;
+
+  tmp_color = G_STRDUP(color);
+  G_FREE(list_entry_background_color[color_code]);
+
+  list_entry_background_color[color_code] = tmp_color;
+  if (*tmp_color == '\0')
+    return NULL;
+  return tmp_color;
 }
 
 
@@ -1576,7 +1873,7 @@ decorate_threadlist_entry(GtkTreeViewColumn *tree_column,
 			  GtkTreeIter *iter, gpointer view)
 {
   guint flags;
-  guint color_code;
+  const char *color;
   GValue fg = { 0, };
   GValue bg = { 0, };
 
@@ -1587,17 +1884,15 @@ decorate_threadlist_entry(GtkTreeViewColumn *tree_column,
 
   if (!GTK_IS_CELL_RENDERER_TOGGLE(renderer))
     {
-      color_code
-	= threadlist_view_decode_visual_param_foreground_color(view, flags);
+      color = threadlist_view_decode_visual_param_foreground(view, flags);
       g_value_init(&fg, G_TYPE_STRING);
-      g_value_set_static_string(&fg, list_entry_foreground_color[color_code]);
+      g_value_set_static_string(&fg, color);
       g_object_set_property((GObject *)renderer, "foreground", &fg);
     }
 
-  color_code =
-    threadlist_view_decode_visual_param_background_color(view, flags);
+  color = threadlist_view_decode_visual_param_background(view, flags);
   g_value_init(&bg, G_TYPE_STRING);
-  g_value_set_static_string(&bg, list_entry_background_color[color_code]);
+  g_value_set_static_string(&bg, color);
   g_object_set_property((GObject *)renderer, "cell_background", &bg);
 
   if (!GTK_IS_CELL_RENDERER_TEXT(renderer))
@@ -1783,10 +2078,14 @@ threadlist_view_append_rank(ThreadlistView *view)
 {
   GtkTreeViewColumn *column;
   GtkCellRenderer *renderer;
+  GValue xalign = { 0, };
 
   g_return_if_fail(IS_THREADLIST_VIEW(view) && view->rank_column == NULL);
 
   renderer = gtk_cell_renderer_text_new();
+  g_value_init(&xalign, G_TYPE_FLOAT);
+  g_value_set_float(&xalign, 0.8);
+  g_object_set_property((GObject *)renderer, "xalign", &xalign);
   column = gtk_tree_view_column_new_with_attributes(_("Rank"), renderer,
 						    "text", THREADLIST_RANK,
 						    NULL);
