@@ -23,37 +23,17 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ugly_gtk2utils.c,v 1.7 2003/11/05 10:22:15 fuyu Exp $
+ * $Id: ugly_gtk2utils.c,v 1.8 2003/11/07 14:27:06 fuyu Exp $
  */
 
 #include "ugly_gtk2utils.h"
 
-#include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <gtk/gtk.h>
 
 #include <stdio.h>
 
 
-void
-ugly_gtk_tree_view_unbind_keys(void)
-{
-  GObjectClass *klass = g_type_class_ref(GTK_TYPE_TREE_VIEW);
-  GtkBindingSet *binding_set = gtk_binding_set_by_class(klass);
-
-  gtk_binding_entry_remove(binding_set, GDK_space, 0);
-  gtk_binding_entry_remove(binding_set, GDK_BackSpace, 0);
-  gtk_binding_entry_remove(binding_set, GDK_f, GDK_CONTROL_MASK);
-  gtk_binding_entry_remove(binding_set, GDK_F, GDK_CONTROL_MASK);
-  gtk_binding_entry_remove(binding_set, GDK_Up, 0);
-  gtk_binding_entry_remove(binding_set, GDK_Down, 0);
-  gtk_binding_entry_remove(binding_set, GDK_p, GDK_CONTROL_MASK);
-  gtk_binding_entry_remove(binding_set, GDK_n, GDK_CONTROL_MASK);
-
-  g_type_class_unref(klass);
-}
-
-
 static gboolean
 delayed_g_object_unref_real(gpointer data)
 {
