@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_async_buffer.c,v 1.10 2003/12/04 08:50:46 fuyu Exp $
+ * $Id: ochusha_async_buffer.c,v 1.11 2003/12/10 15:13:58 fuyu Exp $
  */
 
 #include "config.h"
@@ -923,7 +923,8 @@ ochusha_async_buffer_is_busy(OchushaAsyncBuffer *buffer,
 {
   g_return_val_if_fail(OCHUSHA_IS_ASYNC_BUFFER(buffer), FALSE);
 
-  return buffer->number_of_lock_waiters > 0;
+  return buffer->state != OCHUSHA_ASYNC_BUFFER_OK
+    || buffer->number_of_lock_waiters > 0;
 }
 
 
