@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.h,v 1.5 2003/11/18 21:02:05 fuyu Exp $
+ * $Id: ochusha_thread_2ch.h,v 1.6 2003/11/29 08:52:05 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_THREAD_2CH_H_
@@ -35,6 +35,8 @@
 #include <glib-object.h>
 #include <glib.h>
 
+#include <time.h>
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -64,6 +66,7 @@ struct _OchushaThread2ch
 
   /* responses[n]: (0ϤޤΤ)n+1ܤΥ쥹DAToffset */
   unsigned int *responses;
+  unsigned int responses_length;
 
   char *last_modified;	/* DATեɤ߹߻Last-Modifiedإå */
   char *date;		/* 񤭹߻˻Ȥ */
@@ -76,6 +79,12 @@ struct _OchushaThread2ch
 struct _OchushaThread2chClass
 {
   OchushaBBSThreadClass parent_class;
+
+  char *(*get_base_path)(OchushaThread2ch *thread);
+  char *(*make_post_response_message)(OchushaThread2ch *thread,
+				      const char *from, const char *mail,
+				      const char *message, const char *bbs,
+				      const char *key, time_t time);
 };
 
 
@@ -88,7 +97,6 @@ void ochusha_thread_2ch_set_dat_filename(OchushaThread2ch *thread,
 					 const gchar *dat_filename);
 const char *ochusha_thread_2ch_get_dat_filename(OchushaThread2ch *thread);
 const char *ochusha_thread_2ch_get_dat_url(OchushaThread2ch *thread);
-const char *ochusha_thread_2ch_get_base_url(OchushaThread2ch *thread);
 const char *ochusha_thread_2ch_get_base_path(OchushaThread2ch *thread);
 
 
