@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_jbbs.c,v 1.9 2003/12/11 18:06:24 fuyu Exp $
+ * $Id: ochusha_thread_jbbs.c,v 1.10 2003/12/12 21:42:40 fuyu Exp $
  */
 
 #include "config.h"
@@ -48,16 +48,12 @@
 #include <unistd.h>
 
 
-#if ENABLE_ONIGURUMA
-# if HAVE_ONIG_ONIGURUMA_H
-#  include <onig/oniguruma.h>
-# else
-#  include <oniguruma.h>
-# endif
-# define ENABLE_HTML_TO_DAT_CONVERTER	1
+#if HAVE_ONIG_ONIGURUMA_H
+# include <onig/oniguruma.h>
 #else
-# define ENABLE_HTML_TO_DAT_CONVERTER	0
+# include <oniguruma.h>
 #endif
+#define ENABLE_HTML_TO_DAT_CONVERTER	1
 
 
 #define DEBUG_DAT	0
@@ -912,7 +908,6 @@ ochusha_thread_jbbs_make_post_response_message(OchushaThread2ch *thread_2ch,
 }
 
 
-#if ENABLE_HTML_TO_DAT_CONVERTER
 #define DEFAULT_LINE_BUFFER_SIZE		8192
 #define DEBUG_HTML_TO_DAT_CONVERTER		0
 #define DEBUG_HTML_TO_DAT_CONVERTER_MOST	0
@@ -1505,7 +1500,6 @@ unref_html_buffer(OchushaAsyncBuffer *html_buffer)
 #endif
   g_object_unref(html_buffer);
 }
-#endif	/* ENABLE_HTML_TO_DAT_CONVERTER */
 
 
 /*
@@ -1523,7 +1517,6 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 					  OchushaNetworkBroker *broker,
 					  OchushaNetworkBrokerCacheMode mode)
 {
-#if ENABLE_HTML_TO_DAT_CONVERTER
   OchushaAsyncBuffer *dat_buffer = NULL;
   OchushaAsyncBuffer *html_buffer;
   OchushaBBSThread *thread = OCHUSHA_BBS_THREAD(thread_2ch);
@@ -1683,9 +1676,5 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
   commit_job(job);
 
   return dat_buffer;
-#else
-  fprintf(stderr, "Oniguruma required.\n");
-  return NULL;
-#endif	/* ENABLE_HTML_TO_DAT_CONVERTER */
 }
 
