@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_2ch.c,v 1.19 2003/11/25 18:20:44 fuyu Exp $
+ * $Id: ochusha_utils_2ch.c,v 1.21 2003/11/29 08:52:05 fuyu Exp $
  */
 
 #include "config.h"
@@ -39,29 +39,6 @@
 
 #include <glib.h>
 
-#if ENABLE_REGEXP
-# if HAVE_ONIGURUMA
-#  if HAVE_ONIG_ONIGPOSIX_H
-#    include <onig/onigposix.h>
-#  elif HAVE_ONIGPOSIX_H
-#    include <onigposix.h>
-#  else
-#    error "Use of oniguruma without onig/onigposix.h header isn't considered."
-#  endif
-# else
-#  if HAVE_POSIX_REGEX
-#    if HAVE_SYS_TYPES_H
-#      include <sys/types.h>
-#    endif
-#    if HAVE_REGEX_H
-#      include <regex.h>
-#    else
-#      warning "I don't know suitable header file of your posix regex library."
-#    endif
-#  endif
-# endif
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -180,7 +157,6 @@ parse_thread_path(char *path, char **thread_id_p,
 }
 
 
-#if ENABLE_REGEXP
 static gboolean
 parse_cgi_query(const char *query, char **board_id_p, char **thread_id_p,
 		unsigned int *from_p, unsigned int *to_p)
@@ -190,15 +166,9 @@ parse_cgi_query(const char *query, char **board_id_p, char **thread_id_p,
   char tmp_buf[PATH_MAX];
   unsigned int from = 0;
   unsigned int to = 0;
-  regex_t cgi_query_pattern;
-  regmatch_t match[3];
   char *tmp_pos;
+  CGIQueryKeyTuple tuple;
 
-  if (regcomp(&cgi_query_pattern, "&?([^=]+)=([^&]*)", REG_EXTENDED))
-    {
-      fprintf(stderr, "invalid_regular expression\n");
-      return FALSE;
-    }
 #if 0
   fprintf(stderr, "parse_cgi_query()\n");
   fprintf(stderr, "  query: %s\n", query);
@@ -209,69 +179,56 @@ parse_cgi_query(const char *query, char **board_id_p, char **thread_id_p,
 
   tmp_pos = strchr(query, '?');
   if (tmp_pos != NULL)
-    query = tmp_pos + 1;	/* ?ޤǤФ */
+    query = tmp_pos + 1;	/* ǽ'?'ޤǤФ */
 
-  while (regexec(&cgi_query_pattern, query, 2, match, 0) == 0)
+  while ((tmp_pos = ochusha_utils_find_cgi_query_key(query, &tuple)) != NULL)
     {
-      int match_len = match[0].rm_eo - match[0].rm_so;
-      const char *name_head = query + match[1].rm_so;
-      int name_len = match[1].rm_eo - match[1].rm_so;
-      const char *value_head = query + match[2].rm_so;
-      int value_len = match[2].rm_eo - match[2].rm_so;
-
-      if (g_ascii_strncasecmp("BBS", name_head, name_len) == 0)
+      if (g_ascii_strncasecmp("BBS", tuple.name, tuple.name_len) == 0)
 	{
-	  if (value_len < (PATH_MAX - 1))
+	  if (tuple.value_len < (PATH_MAX - 1))
 	    {
-	      memcpy(board_id, value_head, value_len);
-	      board_id[value_len] = '\0';
+	      memcpy(board_id, tuple.value, tuple.value_len);
+	      board_id[tuple.value_len] = '\0';
 	    }
 	}
-      else if (g_ascii_strncasecmp("KEY", name_head, name_len) == 0)
+      else if (g_ascii_strncasecmp("KEY", tuple.name, tuple.name_len) == 0)
 	{
-	  if (value_len < (PATH_MAX - 1))
+	  if (tuple.value_len < (PATH_MAX - 1))
 	    {
-	      memcpy(thread_id, value_head, value_len);
-	      thread_id[value_len] = '\0';
+	      memcpy(thread_id, tuple.value, tuple.value_len);
+	      thread_id[tuple.value_len] = '\0';
 	    }
 	}
-      else if (g_ascii_strncasecmp("START", name_head, name_len) == 0
-	       || g_ascii_strncasecmp("ST", name_head, name_len) == 0)
+      else if (g_ascii_strncasecmp("START", tuple.name, tuple.name_len) == 0
+	       || g_ascii_strncasecmp("ST", tuple.name, tuple.name_len) == 0)
 	{
-	  if (value_len > 0 && value_len < (PATH_MAX - 1))
+	  if (tuple.value_len > 0 && tuple.value_len < (PATH_MAX - 1))
 	    {
-	      memcpy(tmp_buf, value_head, value_len);
-	      tmp_buf[value_len] = '\0';
+	      memcpy(tmp_buf, tuple.value, tuple.value_len);
+	      tmp_buf[tuple.value_len] = '\0';
 	      if (sscanf(tmp_buf, "%d", &from) != 1)
 		from = 0;
 	    }
 	}
-      else if (g_ascii_strncasecmp("END", name_head, name_len) == 0
-	       || g_ascii_strncasecmp("TO", name_head, name_len) == 0)
+      else if (g_ascii_strncasecmp("END", tuple.name, tuple.name_len) == 0
+	       || g_ascii_strncasecmp("TO", tuple.name, tuple.name_len) == 0)
 	{
-	  if (value_len > 0 && value_len < (PATH_MAX - 1))
+	  if (tuple.value_len > 0 && tuple.value_len < (PATH_MAX - 1))
 	    {
-	      memcpy(tmp_buf, value_head, value_len);
-	      tmp_buf[value_len] = '\0';
+	      memcpy(tmp_buf, tuple.value, tuple.value_len);
+	      tmp_buf[tuple.value_len] = '\0';
 	      if (sscanf(tmp_buf, "%d", &to) != 1)
 		to = 0;
 	    }
 	}
 
-      query += match_len;
+      query = tmp_pos;
     }
 
   if (from == 0 && to != 0)
     from = to;
   else if (to == 0 && from != 0)
     to = from;
-#if 0
-  else if (from == 0 && to == 0)
-    {
-      from = 1;
-      to = 1;
-    }
-#endif
 
   if (from_p != NULL)
     *from_p = from;
@@ -304,11 +261,8 @@ parse_cgi_query(const char *query, char **board_id_p, char **thread_id_p,
   fprintf(stderr, "  from=%d, to=%d\n", from, to);
 #endif
 
-  regfree(&cgi_query_pattern);
-
   return board_id[0] != '\0';
 }
-#endif
 
 
 /*
@@ -428,7 +382,6 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
 		board_url[0] = '\0';
 	    }
 	}
-#if ENABLE_REGEXP
       else if (path_pos[13] == '?')
 	{
 	  if (parse_cgi_query(path_pos + 14, &board_id, &thread_id,
@@ -442,7 +395,6 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
 #endif
 	    }
 	}
-#endif
     }
   else if (g_ascii_strcasecmp(server, "jbbs.shitaraba.com") == 0)
     {
@@ -507,7 +459,6 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
 		    }
 		}
 	    }
-#if ENABLE_REGEXP
 	  else if (tmp_pos[12] == '?')
 	    {
 	      /*
@@ -523,10 +474,8 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
 		    board_url[0] = '\0';
 		}
 	    }
-#endif
 	}
     }
-#if ENABLE_REGEXP
   else if (strncmp(path_pos, "bbs/read.pl?", 12) == 0)
     {
       bbs_type = OCHUSHA_BBS_TYPE_MACHIBBS;
@@ -555,19 +504,22 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
 #endif
 	}
     }
-#endif
   else if (strncmp(path_pos, "bbs/read.cgi", 12) == 0)
     {
       if (path_pos[12] == '/')
 	{
 	  /* JBBSФ../../bbs/read.cgi/ϤURL⤳ˤ뤫*/
 	  char *slash_pos = strchr(path_pos + 13, '/');
+#if 0
 	  fprintf(stderr, "slash_pos=%p\n", slash_pos);
+#endif
 	  if (slash_pos != NULL)
 	    {
 	      char *tmp_pos = strchr(slash_pos + 1, '/');
 	      int len;
+#if 0
 	      fprintf(stderr, "tmp_pos=%p\n", tmp_pos);
+#endif
 	      if (tmp_pos != NULL && tmp_pos < (slash_pos + 10))
 		{
 		  /* ɥۥåʵ§ġġ*/
@@ -590,9 +542,11 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
 		{
 		  if (board_id == NULL)
 		    board_id = G_STRDUP(tmp_buf);
+#if 0
 		  fprintf(stderr, "board_id=\"%s\"\n", board_id);
 		  fprintf(stderr, "tmp_buf=\"%s\"\n", tmp_buf);
 		  fprintf(stderr, "board_url=\"%s\"\n", board_url);
+#endif
 		  parse_thread_path(slash_pos + 1, &thread_id, &from, &to);
 		}
 	      else
@@ -605,7 +559,6 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
 #endif
 	    }
 	}
-#if ENABLE_REGEXP
       else if (path_pos[12] == '?')
 	{
 	  if (parse_cgi_query(path_pos + 13, &board_id, &thread_id,
@@ -621,7 +574,6 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
 #endif
 	    }
 	}
-#endif
     }
   else
     {
