@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: utils.h,v 1.6 2003/11/02 22:01:15 fuyu Exp $
+ * $Id: utils.h,v 1.8 2003/11/30 16:43:56 fuyu Exp $
  */
 
 #ifndef _UTILS_H_
@@ -55,6 +55,7 @@ size_t cp932_to_utf8_helper(const char **inbuf, size_t *inbytesleft,
 
 gchar *wipe_string(const gchar *string);
 
+char *ochusha_utils_url_extract_scheme(const char *url);
 char *ochusha_utils_url_extract_http_server(const char *url);
 char *ochusha_utils_url_extract_http_absolute_path(const char *url);
 char *ochusha_utils_url_encode_string(const char *src);
@@ -62,9 +63,22 @@ char *ochusha_utils_url_decode_string(const char *src);
 
 time_t ochusha_utils_get_utc_time(const char *date_string);
 
+typedef struct _CGIQueryKeyTuple
+{
+  const char *name;
+  int name_len;
+  const char *value;
+  int value_len;
+} CGIQueryKeyTuple;
+
+char *ochusha_utils_find_cgi_query_key(const char *src,
+				       CGIQueryKeyTuple *result);
+
 int mkdir_p(const char *path);
 
 
+
+
 #if TRACE_MEMORY_USAGE
 /*
  * ꡼åϢ
